
/* #line 1 "./ragel/tsdp_parser_header_S.rl" */
/*
* Copyright (C) 2009 Mamadou Diop.
*
* Contact: Mamadou Diop <diopmamadou(at)doubango.org>
*	
* This file is part of Open Source Doubango Framework.
*
* DOUBANGO is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*	
* DOUBANGO is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*	
* You should have received a copy of the GNU General Public License
* along with DOUBANGO.
*
*/

/**@file tsdp_header_S.c
 * @brief SDP "s=" header (Session Name).
 *
 * @author Mamadou Diop <diopmamadou(at)doubango.org>
 *
 * @date Created: Sat Nov 8 16:54:58 2009 mdiop
 */
#include "tinysdp/headers/tsdp_header_S.h"

#include "tsk_debug.h"
#include "tsk_memory.h"
#include "tsk_string.h"

#include <string.h>

/***********************************
*	Ragel state machine.
*/

/* #line 60 "./ragel/tsdp_parser_header_S.rl" */



tsdp_header_S_t* tsdp_header_S_create(const char* value)
{
	return tsk_object_new(TSDP_HEADER_S_VA_ARGS(value));
}
tsdp_header_S_t* tsdp_header_S_create_null()
{
	return tsdp_header_S_create(tsk_null);
}

int tsdp_header_S_tostring(const tsdp_header_t* header, tsk_buffer_t* output)
{
	if(header)
	{
		const tsdp_header_S_t *S = (const tsdp_header_S_t *)header;
		if(S->value){
			tsk_buffer_append(output, S->value, tsk_strlen(S->value));
		}
		return 0;
	}

	return -1;
}

tsdp_header_t* tsdp_header_S_clone(const tsdp_header_t* header)
{
	if(header){
		const tsdp_header_S_t *S = (const tsdp_header_S_t *)header;
		return (tsdp_header_t*)tsdp_header_S_create(S->value);
	}
	return tsk_null;
}

tsdp_header_S_t *tsdp_header_S_parse(const char *data, tsk_size_t size)
{
	int cs = 0;
	const char *p = data;
	const char *pe = p + size;
	const char *eof = pe;
	tsdp_header_S_t *hdr_S = tsdp_header_S_create_null();
	
	const char *tag_start;

	
/* #line 92 "./src/headers/tsdp_header_S.c" */
static const char _tsdp_machine_parser_header_S_actions[] = {
	0, 1, 0, 1, 1, 2, 0, 1
	
};

static const char _tsdp_machine_parser_header_S_key_offsets[] = {
	0, 0, 1, 3, 4, 6, 7
};

static const char _tsdp_machine_parser_header_S_trans_keys[] = {
	115, 32, 61, 10, 13, 32, 13, 0
};

static const char _tsdp_machine_parser_header_S_single_lengths[] = {
	0, 1, 2, 1, 2, 1, 0
};

static const char _tsdp_machine_parser_header_S_range_lengths[] = {
	0, 0, 0, 0, 0, 0, 0
};

static const char _tsdp_machine_parser_header_S_index_offsets[] = {
	0, 0, 2, 5, 7, 10, 12
};

static const char _tsdp_machine_parser_header_S_trans_targs[] = {
	2, 0, 2, 4, 0, 6, 0, 3, 
	4, 5, 3, 5, 0, 0
};

static const char _tsdp_machine_parser_header_S_trans_actions[] = {
	0, 0, 0, 0, 0, 0, 0, 5, 
	0, 1, 3, 0, 0, 0
};

static const char _tsdp_machine_parser_header_S_eof_actions[] = {
	0, 0, 0, 0, 5, 3, 0
};

static const int tsdp_machine_parser_header_S_start = 1;
static const int tsdp_machine_parser_header_S_first_final = 4;
static const int tsdp_machine_parser_header_S_error = 0;

static const int tsdp_machine_parser_header_S_en_main = 1;


/* #line 106 "./ragel/tsdp_parser_header_S.rl" */
	
/* #line 141 "./src/headers/tsdp_header_S.c" */
	{
	cs = tsdp_machine_parser_header_S_start;
	}

/* #line 107 "./ragel/tsdp_parser_header_S.rl" */
	
/* #line 148 "./src/headers/tsdp_header_S.c" */
	{
	int _klen;
	unsigned int _trans;
	const char *_acts;
	unsigned int _nacts;
	const char *_keys;

	if ( p == pe )
		goto _test_eof;
	if ( cs == 0 )
		goto _out;
_resume:
	_keys = _tsdp_machine_parser_header_S_trans_keys + _tsdp_machine_parser_header_S_key_offsets[cs];
	_trans = _tsdp_machine_parser_header_S_index_offsets[cs];

	_klen = _tsdp_machine_parser_header_S_single_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + _klen - 1;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + ((_upper-_lower) >> 1);
			if ( (*p) < *_mid )
				_upper = _mid - 1;
			else if ( (*p) > *_mid )
				_lower = _mid + 1;
			else {
				_trans += (_mid - _keys);
				goto _match;
			}
		}
		_keys += _klen;
		_trans += _klen;
	}

	_klen = _tsdp_machine_parser_header_S_range_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + (_klen<<1) - 2;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + (((_upper-_lower) >> 1) & ~1);
			if ( (*p) < _mid[0] )
				_upper = _mid - 2;
			else if ( (*p) > _mid[1] )
				_lower = _mid + 2;
			else {
				_trans += ((_mid - _keys)>>1);
				goto _match;
			}
		}
		_trans += _klen;
	}

_match:
	cs = _tsdp_machine_parser_header_S_trans_targs[_trans];

	if ( _tsdp_machine_parser_header_S_trans_actions[_trans] == 0 )
		goto _again;

	_acts = _tsdp_machine_parser_header_S_actions + _tsdp_machine_parser_header_S_trans_actions[_trans];
	_nacts = (unsigned int) *_acts++;
	while ( _nacts-- > 0 )
	{
		switch ( *_acts++ )
		{
	case 0:
/* #line 47 "./ragel/tsdp_parser_header_S.rl" */
	{
		tag_start = p;
	}
	break;
	case 1:
/* #line 51 "./ragel/tsdp_parser_header_S.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_S->value);
	}
	break;
/* #line 233 "./src/headers/tsdp_header_S.c" */
		}
	}

_again:
	if ( cs == 0 )
		goto _out;
	if ( ++p != pe )
		goto _resume;
	_test_eof: {}
	if ( p == eof )
	{
	const char *__acts = _tsdp_machine_parser_header_S_actions + _tsdp_machine_parser_header_S_eof_actions[cs];
	unsigned int __nacts = (unsigned int) *__acts++;
	while ( __nacts-- > 0 ) {
		switch ( *__acts++ ) {
	case 0:
/* #line 47 "./ragel/tsdp_parser_header_S.rl" */
	{
		tag_start = p;
	}
	break;
	case 1:
/* #line 51 "./ragel/tsdp_parser_header_S.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_S->value);
	}
	break;
/* #line 261 "./src/headers/tsdp_header_S.c" */
		}
	}
	}

	_out: {}
	}

/* #line 108 "./ragel/tsdp_parser_header_S.rl" */
	
	if( cs < 
/* #line 272 "./src/headers/tsdp_header_S.c" */
4
/* #line 109 "./ragel/tsdp_parser_header_S.rl" */
 ){
		TSK_OBJECT_SAFE_FREE(hdr_S);
	}
	
	return hdr_S;
}







//========================================================
//	S header object definition
//

static tsk_object_t* tsdp_header_S_ctor(tsk_object_t *self, va_list * app)
{
	tsdp_header_S_t *S = self;
	if(S){
		TSDP_HEADER(S)->type = tsdp_htype_S;
		TSDP_HEADER(S)->tostring = tsdp_header_S_tostring;
		TSDP_HEADER(S)->clone = tsdp_header_S_clone;
		TSDP_HEADER(S)->rank = TSDP_HTYPE_S_RANK;
		
		S->value = tsk_strdup(va_arg(*app, const char*));
	}
	else{
		TSK_DEBUG_ERROR("Failed to create new S header.");
	}
	return self;
}

static tsk_object_t* tsdp_header_S_dtor(tsk_object_t *self)
{
	tsdp_header_S_t *S = self;
	if(S){
		TSK_FREE(S->value);
	}
	else{
		TSK_DEBUG_ERROR("Null S header.");
	}

	return self;
}
static int tsdp_header_S_cmp(const tsk_object_t *obj1, const tsk_object_t *obj2)
{
	if(obj1 && obj2){
		return tsdp_header_rank_cmp(obj1, obj2);
	}
	else{
		return -1;
	}
}

static const tsk_object_def_t tsdp_header_S_def_s = 
{
	sizeof(tsdp_header_S_t),
	tsdp_header_S_ctor,
	tsdp_header_S_dtor,
	tsdp_header_S_cmp
};

const tsk_object_def_t *tsdp_header_S_def_t = &tsdp_header_S_def_s;
