
/* #line 1 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
/*
* Copyright (C) 2009 Mamadou Diop.
*
* Contact: Mamadou Diop <diopmamadou(at)doubango.org>
*	
* This file is part of Open Source Doubango Framework.
*
* DOUBANGO is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*	
* DOUBANGO is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*	
* You should have received a copy of the GNU General Public License
* along with DOUBANGO.
*
*/

/**@file thttp_header_WWW_Authenticate.c
 * @brief HTTP WWW-Authenticate header.
 *
 * @author Mamadou Diop <diopmamadou(at)doubango.org>
 *
 * @date Created: Sat Nov 8 16:54:58 2009 mdiop
 */
#include "tinyhttp/headers/thttp_header_WWW_Authenticate.h"

#include "tinyhttp/parsers/thttp_parser_url.h"

#include "tsk_debug.h"
#include "tsk_memory.h"
#include "tsk_time.h"

#include <string.h>


/***********************************
*	Ragel state machine.
*/

/* #line 129 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */


thttp_header_WWW_Authenticate_t* thttp_header_www_authenticate_create()
{
	return tsk_object_new(thttp_header_WWW_Authenticate_def_t);
}

int thttp_header_WWW_Authenticate_tostring(const thttp_header_t* header, tsk_buffer_t* output)
{
	if(header){
		const thttp_header_WWW_Authenticate_t *WWW_Authenticate = (const thttp_header_WWW_Authenticate_t*)header;
		if(WWW_Authenticate && WWW_Authenticate->scheme){
			return tsk_buffer_append_2(output, "%s realm=\"%s\"%s%s%s%s%s%s%s%s%s%s%s%s,stale=%s%s%s", 
				WWW_Authenticate->scheme,
				WWW_Authenticate->realm ? WWW_Authenticate->realm : "",
				
				WWW_Authenticate->domain ? ",domain=\"" : "",
				WWW_Authenticate->domain ? WWW_Authenticate->domain : "",
				WWW_Authenticate->domain ? "\"" : "",
				
				
				WWW_Authenticate->qop ? ",qop=\"" : "",
				WWW_Authenticate->qop ? WWW_Authenticate->qop : "",
				WWW_Authenticate->qop ? "\"" : "",


				WWW_Authenticate->nonce ? ",nonce=\"" : "",
				WWW_Authenticate->nonce ? WWW_Authenticate->nonce : "",
				WWW_Authenticate->nonce ? "\"" : "",

				WWW_Authenticate->opaque ? ",opaque=\"" : "",
				WWW_Authenticate->opaque ? WWW_Authenticate->opaque : "",
				WWW_Authenticate->opaque ? "\"" : "",

				WWW_Authenticate->stale ? "TRUE" : "FALSE",

				WWW_Authenticate->algorithm ? ",algorithm=" : "",
				WWW_Authenticate->algorithm ? WWW_Authenticate->algorithm : ""
				);
		}
	}
	return -1;
}

/**@ingroup thttp_header_group
*/
thttp_header_WWW_Authenticate_t *thttp_header_WWW_Authenticate_parse(const char *data, tsk_size_t size)
{
	int cs = 0;
	const char *p = data;
	const char *pe = p + size;
	const char *eof = pe;
	thttp_header_WWW_Authenticate_t *hdr_WWW_Authenticate = thttp_header_www_authenticate_create();
	
	const char *tag_start;

	
/* #line 106 "./src/headers/thttp_header_WWW_Authenticate.c" */
static const char _thttp_machine_parser_header_WWW_Authenticate_actions[] = {
	0, 1, 0, 1, 1, 1, 2, 1, 
	3, 1, 4, 1, 5, 1, 6, 1, 
	7, 1, 8, 1, 9, 1, 10, 1, 
	11, 1, 12, 1, 13, 2, 0, 6, 
	2, 0, 7, 2, 0, 8, 2, 0, 
	11, 2, 0, 12, 2, 5, 0, 2, 
	6, 0, 2, 7, 0, 2, 8, 0, 
	2, 12, 0
};

static const short _thttp_machine_parser_header_WWW_Authenticate_key_offsets[] = {
	0, 0, 4, 6, 8, 10, 12, 13, 
	15, 17, 19, 21, 23, 25, 27, 29, 
	31, 33, 35, 37, 40, 47, 48, 49, 
	56, 59, 62, 65, 68, 71, 102, 119, 
	124, 155, 174, 193, 209, 213, 221, 225, 
	230, 238, 247, 256, 265, 268, 276, 286, 
	289, 293, 296, 300, 303, 307, 311, 313, 
	317, 319, 323, 325, 335, 345, 354, 363, 
	372, 381, 384, 391, 401, 411, 421, 424, 
	428, 432, 434, 436, 455, 474, 493, 512, 
	531, 550, 569, 586, 591, 610, 626, 645, 
	664, 683, 702, 721, 738, 743, 762, 770, 
	778, 780, 786, 795, 831, 853, 863, 886, 
	907, 916, 931, 947, 963, 979, 989, 1004, 
	1021, 1031, 1042, 1052, 1063, 1073, 1084, 1095, 
	1104, 1115, 1124, 1135, 1144, 1161, 1178, 1194, 
	1210, 1226, 1242, 1252, 1266, 1283, 1300, 1317, 
	1327, 1338, 1349, 1358, 1367, 1391, 1415, 1439, 
	1463, 1487, 1511, 1535, 1559, 1581, 1591, 1614, 
	1635, 1659, 1683, 1707, 1731, 1755, 1777, 1787, 
	1810, 1819, 1828, 1864, 1886, 1896, 1919, 1940, 
	1955, 1971, 1987, 2003, 2013, 2028, 2045, 2055, 
	2066, 2076, 2087, 2097, 2108, 2119, 2128, 2137, 
	2148, 2157, 2168, 2177, 2194, 2211, 2227, 2243, 
	2259, 2275, 2285, 2299, 2316, 2333, 2350, 2360, 
	2371, 2382, 2391, 2400, 2424, 2448, 2472, 2496, 
	2520, 2544, 2568, 2592, 2614, 2624, 2647, 2668, 
	2692, 2716, 2740, 2764, 2788, 2810, 2820, 2843, 
	2867, 2891, 2915, 2939, 2961, 2971, 2994, 3003, 
	3011, 3015, 3020, 3056, 3078, 3088, 3111, 3132, 
	3141, 3156, 3172, 3188, 3204, 3214, 3229, 3246, 
	3256, 3267, 3277, 3288, 3298, 3309, 3320, 3329, 
	3338, 3349, 3358, 3369, 3378, 3395, 3412, 3428, 
	3444, 3460, 3476, 3486, 3500, 3517, 3534, 3551, 
	3561, 3572, 3583, 3592, 3601, 3625, 3649, 3673, 
	3697, 3721, 3745, 3769, 3793, 3815, 3825, 3848, 
	3869, 3893, 3917, 3941, 3965, 3989, 4011, 4021, 
	4044, 4053, 4077, 4101, 4125, 4149, 4171, 4181, 
	4204, 4213, 4237, 4261, 4285, 4309, 4333, 4355, 
	4365, 4388, 4397, 4405, 4409, 4414, 4423, 4459, 
	4481, 4491, 4514, 4535, 4544, 4559, 4575, 4591, 
	4607, 4617, 4632, 4649, 4659, 4670, 4680, 4691, 
	4701, 4712, 4723, 4732, 4741, 4752, 4761, 4772, 
	4781, 4798, 4815, 4831, 4847, 4863, 4879, 4889, 
	4903, 4920, 4937, 4954, 4964, 4975, 4986, 4995, 
	5004, 5028, 5052, 5076, 5100, 5124, 5148, 5172, 
	5196, 5218, 5228, 5251, 5272, 5296, 5320, 5344, 
	5368, 5392, 5414, 5424, 5447, 5456, 5480, 5504, 
	5528, 5552, 5574, 5584, 5607, 5616, 5640, 5664, 
	5688, 5712, 5736, 5758, 5768, 5791, 5800, 5824, 
	5848, 5870, 5880, 5903, 5912, 5920, 5922, 5928, 
	5937, 5973, 5995, 6005, 6028, 6049, 6064, 6080, 
	6096, 6112, 6122, 6137, 6154, 6164, 6175, 6185, 
	6196, 6206, 6217, 6228, 6237, 6246, 6257, 6266, 
	6277, 6286, 6303, 6320, 6336, 6352, 6368, 6384, 
	6394, 6408, 6425, 6442, 6459, 6469, 6480, 6491, 
	6500, 6509, 6533, 6557, 6581, 6605, 6629, 6653, 
	6677, 6701, 6723, 6733, 6756, 6777, 6801, 6825, 
	6849, 6873, 6897, 6919, 6929, 6952, 6961, 6985, 
	7009, 7033, 7057, 7079, 7089, 7112, 7136, 7160, 
	7184, 7208, 7232, 7254, 7264, 7287, 7311, 7335, 
	7357, 7367, 7390, 7414, 7438, 7462, 7486, 7508, 
	7518, 7541, 7550, 7558, 7562, 7567, 7603, 7625, 
	7635, 7658, 7679, 7688, 7703, 7719, 7735, 7751, 
	7761, 7776, 7793, 7803, 7814, 7824, 7835, 7845, 
	7856, 7867, 7876, 7885, 7896, 7905, 7916, 7925, 
	7942, 7959, 7975, 7991, 8007, 8023, 8033, 8047, 
	8064, 8081, 8098, 8108, 8119, 8130, 8139, 8148, 
	8172, 8196, 8220, 8244, 8268, 8292, 8316, 8340, 
	8362, 8372, 8395, 8416, 8440, 8464, 8488, 8512, 
	8536, 8558, 8568, 8591, 8600, 8624, 8648, 8672, 
	8696, 8718, 8728, 8751, 8760, 8784, 8808, 8832, 
	8856, 8880, 8902, 8912, 8935, 8944, 8968, 8992, 
	9014, 9024, 9047, 9056, 9080, 9104, 9128, 9152, 
	9174, 9184, 9207, 9216, 9240, 9264, 9288, 9312, 
	9334, 9344, 9371, 9394, 9417, 9440, 9463, 9472, 
	9495, 9518, 9542, 9566, 9590, 9614, 9636, 9646, 
	9673, 9696, 9719, 9742, 9765, 9774, 9797, 9820, 
	9844, 9868, 9892, 9916, 9938, 9948, 9971, 9980, 
	10004, 10028, 10052, 10076, 10098, 10108, 10135, 10158, 
	10181, 10204, 10227, 10236, 10259, 10282, 10306, 10330, 
	10352, 10362, 10385, 10394, 10418, 10442, 10466, 10490, 
	10512, 10522, 10545, 10554, 10578, 10602, 10626, 10650, 
	10672, 10682, 10709, 10732, 10755, 10778, 10801, 10810, 
	10833, 10856, 10880, 10904, 10928, 10952, 10976, 10998, 
	11008, 11031, 11055, 11079, 11101, 11111, 11134, 11143, 
	11167, 11191, 11215, 11239, 11261, 11271, 11294, 11318, 
	11342, 11366, 11390, 11412, 11422, 11449, 11472, 11495, 
	11518, 11541, 11550, 11573, 11596, 11620, 11644, 11668, 
	11692, 11714, 11724, 11747, 11771, 11795, 11819, 11843, 
	11867, 11889, 11899, 11922, 11946, 11970, 11992, 12002, 
	12025, 12034, 12058, 12082, 12106, 12130, 12152, 12162, 
	12185, 12209, 12233, 12257, 12281, 12303, 12313, 12340, 
	12363, 12386, 12409, 12432, 12441, 12464, 12487, 12506, 
	12525, 12544, 12563, 12580, 12585, 12604, 12623, 12642, 
	12661, 12680, 12699, 12716, 12721, 12740, 12759, 12778, 
	12795, 12800, 12819, 12827, 12846, 12865, 12884, 12903, 
	12920, 12925, 12944, 12963, 12982, 13001, 13020, 13037, 
	13042, 13065, 13083, 13101, 13119, 13137, 13141, 13159, 
	13177, 13196, 13215, 13234, 13253, 13272, 13289, 13294, 
	13313, 13321, 13329, 13335, 13344, 13380, 13402, 13412, 
	13435, 13456, 13471, 13487, 13503, 13519, 13529, 13544, 
	13561, 13571, 13582, 13592, 13603, 13613, 13624, 13635, 
	13644, 13653, 13664, 13673, 13684, 13693, 13710, 13727, 
	13743, 13759, 13775, 13791, 13801, 13815, 13832, 13849, 
	13866, 13876, 13887, 13898, 13907, 13916, 13940, 13964, 
	13988, 14012, 14036, 14060, 14084, 14108, 14130, 14140, 
	14163, 14184, 14208, 14232, 14256, 14280, 14304, 14326, 
	14336, 14359, 14368, 14392, 14416, 14440, 14464, 14486, 
	14496, 14519, 14543, 14567, 14591, 14615, 14639, 14661, 
	14671, 14694, 14718, 14742, 14764, 14774, 14797, 14806, 
	14830, 14854, 14878, 14902, 14924, 14934, 14957, 14981, 
	15005, 15029, 15053, 15075, 15085, 15112, 15135, 15158, 
	15181, 15204, 15213, 15236, 15259, 15278, 15297, 15314, 
	15319, 15338, 15346, 15354, 15360, 15363, 15366, 15369, 
	15372, 15375, 15378, 15380, 15383, 15385, 15387
};

static const char _thttp_machine_parser_header_WWW_Authenticate_trans_keys[] = {
	80, 87, 112, 119, 82, 114, 79, 111, 
	88, 120, 89, 121, 45, 65, 97, 85, 
	117, 84, 116, 72, 104, 69, 101, 78, 
	110, 84, 116, 73, 105, 67, 99, 65, 
	97, 84, 116, 69, 101, 9, 32, 58, 
	9, 13, 32, 66, 68, 98, 100, 13, 
	10, 9, 13, 32, 66, 68, 98, 100, 
	13, 65, 97, 13, 83, 115, 13, 73, 
	105, 13, 67, 99, 9, 13, 32, 9, 
	13, 32, 33, 37, 39, 65, 68, 78, 
	79, 81, 82, 83, 97, 100, 110, 111, 
	113, 114, 115, 126, 42, 43, 45, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 44, 61, 9, 13, 32, 33, 
	37, 39, 65, 68, 78, 79, 81, 82, 
	83, 97, 100, 110, 111, 113, 114, 115, 
	126, 42, 43, 45, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 76, 108, 126, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 91, 126, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 44, 13, 34, 92, 
	127, 0, 8, 10, 31, 9, 13, 32, 
	44, 13, 0, 9, 11, 127, 13, 58, 
	48, 57, 65, 70, 97, 102, 13, 58, 
	93, 48, 57, 65, 70, 97, 102, 13, 
	58, 93, 48, 57, 65, 70, 97, 102, 
	13, 58, 93, 48, 57, 65, 70, 97, 
	102, 13, 58, 93, 13, 58, 48, 57, 
	65, 70, 97, 102, 13, 46, 58, 93, 
	48, 57, 65, 70, 97, 102, 13, 48, 
	57, 13, 46, 48, 57, 13, 48, 57, 
	13, 46, 48, 57, 13, 48, 57, 13, 
	93, 48, 57, 13, 93, 48, 57, 13, 
	93, 13, 46, 48, 57, 13, 46, 13, 
	46, 48, 57, 13, 46, 13, 46, 58, 
	93, 48, 57, 65, 70, 97, 102, 13, 
	46, 58, 93, 48, 57, 65, 70, 97, 
	102, 13, 58, 93, 48, 57, 65, 70, 
	97, 102, 13, 58, 93, 48, 57, 65, 
	70, 97, 102, 13, 58, 93, 48, 57, 
	65, 70, 97, 102, 13, 58, 93, 48, 
	57, 65, 70, 97, 102, 13, 58, 93, 
	13, 48, 57, 65, 70, 97, 102, 13, 
	46, 58, 93, 48, 57, 65, 70, 97, 
	102, 13, 46, 58, 93, 48, 57, 65, 
	70, 97, 102, 13, 46, 58, 93, 48, 
	57, 65, 70, 97, 102, 13, 48, 57, 
	13, 46, 48, 57, 13, 46, 48, 57, 
	13, 46, 13, 58, 9, 13, 32, 33, 
	37, 39, 44, 61, 71, 103, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 79, 
	111, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 82, 114, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 61, 73, 105, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	84, 116, 126, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 72, 104, 126, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 77, 109, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 44, 61, 9, 
	13, 32, 33, 34, 37, 39, 91, 126, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 79, 111, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 61, 77, 109, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	65, 97, 126, 42, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 73, 105, 126, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 78, 110, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 44, 61, 9, 
	13, 32, 33, 34, 37, 39, 91, 126, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 13, 34, 92, 127, 0, 8, 
	10, 31, 13, 34, 92, 127, 0, 8, 
	10, 31, 13, 34, 13, 34, 0, 9, 
	11, 127, 9, 13, 32, 34, 44, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 65, 68, 78, 79, 81, 82, 
	83, 92, 97, 100, 110, 111, 113, 114, 
	115, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 66, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	92, 127, 0, 31, 13, 34, 58, 92, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 13, 34, 58, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 46, 58, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 92, 93, 127, 0, 8, 10, 31, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 58, 92, 93, 127, 
	0, 8, 10, 31, 13, 34, 92, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 46, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 46, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 46, 58, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 13, 34, 
	58, 92, 127, 0, 8, 10, 31, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	76, 92, 108, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	71, 92, 103, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	79, 92, 111, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	82, 92, 114, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	73, 92, 105, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	84, 92, 116, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	72, 92, 104, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	77, 92, 109, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 79, 92, 111, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 77, 92, 109, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 65, 92, 97, 126, 
	127, 0, 31, 42, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 73, 92, 105, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 78, 92, 110, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 61, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 91, 92, 126, 127, 0, 31, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 92, 
	127, 0, 31, 9, 13, 32, 34, 44, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 65, 68, 78, 79, 81, 
	82, 83, 92, 97, 100, 110, 111, 113, 
	114, 115, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 13, 34, 58, 92, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 13, 34, 58, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 46, 58, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 92, 93, 127, 0, 8, 10, 31, 
	9, 13, 32, 34, 44, 92, 127, 0, 
	31, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 46, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 58, 92, 93, 
	127, 0, 8, 10, 31, 13, 34, 92, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 46, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 46, 58, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 48, 57, 13, 34, 
	46, 92, 127, 0, 8, 10, 31, 13, 
	34, 58, 92, 127, 0, 8, 10, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 76, 92, 108, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 71, 92, 103, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 82, 92, 114, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 73, 92, 105, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 84, 92, 116, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 72, 92, 104, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 77, 92, 109, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 79, 92, 111, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 77, 92, 109, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 65, 92, 97, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	66, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 73, 92, 105, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 78, 92, 110, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 79, 92, 111, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 78, 92, 110, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 67, 92, 99, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 69, 92, 101, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 61, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 91, 92, 126, 127, 0, 31, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 92, 
	127, 0, 31, 13, 34, 92, 127, 0, 
	8, 10, 31, 9, 13, 32, 44, 13, 
	0, 9, 11, 127, 9, 13, 32, 33, 
	34, 37, 39, 65, 68, 78, 79, 81, 
	82, 83, 92, 97, 100, 110, 111, 113, 
	114, 115, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 13, 34, 58, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 13, 34, 
	58, 92, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 92, 93, 127, 0, 8, 10, 
	31, 9, 13, 32, 34, 44, 92, 127, 
	0, 31, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 13, 34, 
	46, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 13, 34, 46, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 46, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 58, 92, 
	93, 127, 0, 8, 10, 31, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	65, 70, 97, 102, 13, 34, 46, 58, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 65, 70, 97, 102, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	46, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	13, 34, 58, 92, 127, 0, 8, 10, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 76, 92, 108, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 71, 92, 103, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 79, 92, 111, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 82, 92, 114, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 73, 92, 105, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 84, 92, 116, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 72, 92, 104, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 77, 92, 109, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 61, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	91, 92, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 79, 92, 
	111, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 77, 92, 
	109, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 65, 92, 
	97, 126, 127, 0, 31, 42, 46, 48, 
	57, 66, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 73, 92, 
	105, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 78, 92, 
	110, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 91, 92, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 79, 92, 
	111, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 78, 92, 
	110, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 67, 92, 
	99, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 69, 92, 
	101, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 91, 92, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 80, 92, 
	112, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 65, 92, 
	97, 126, 127, 0, 31, 42, 46, 48, 
	57, 66, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 81, 92, 
	113, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 85, 92, 
	117, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 69, 92, 
	101, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 91, 92, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 13, 34, 92, 
	127, 0, 8, 10, 31, 9, 13, 32, 
	44, 13, 0, 9, 11, 127, 9, 13, 
	32, 34, 44, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 65, 68, 
	78, 79, 81, 82, 83, 92, 97, 100, 
	110, 111, 113, 114, 115, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 61, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 91, 92, 126, 127, 0, 31, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	13, 34, 58, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 58, 92, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 48, 57, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 93, 127, 
	0, 8, 10, 31, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	46, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 46, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 13, 34, 58, 92, 127, 
	0, 8, 10, 31, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 76, 92, 108, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 71, 92, 103, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 79, 92, 111, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 82, 92, 114, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 73, 92, 105, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 84, 92, 116, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 72, 92, 104, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 77, 92, 109, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 77, 92, 109, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 73, 92, 105, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 67, 92, 99, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 81, 92, 113, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 85, 92, 117, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	13, 34, 92, 127, 0, 8, 10, 31, 
	13, 34, 13, 34, 0, 9, 11, 127, 
	9, 13, 32, 34, 44, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	65, 68, 78, 79, 81, 82, 83, 92, 
	97, 100, 110, 111, 113, 114, 115, 126, 
	127, 0, 31, 42, 43, 45, 46, 48, 
	57, 66, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 91, 92, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 13, 34, 58, 92, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 13, 34, 58, 92, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 92, 127, 
	0, 8, 10, 31, 48, 57, 13, 34, 
	92, 93, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 92, 93, 
	127, 0, 8, 10, 31, 9, 13, 32, 
	34, 44, 92, 127, 0, 31, 13, 34, 
	46, 92, 127, 0, 8, 10, 31, 48, 
	57, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 13, 34, 
	46, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 46, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 58, 92, 93, 127, 0, 8, 
	10, 31, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 46, 58, 92, 93, 
	127, 0, 8, 10, 31, 48, 57, 65, 
	70, 97, 102, 13, 34, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 13, 34, 58, 92, 
	127, 0, 8, 10, 31, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 76, 92, 
	108, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 71, 92, 
	103, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 79, 92, 
	111, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 82, 92, 
	114, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 73, 92, 
	105, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 84, 92, 
	116, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 72, 92, 
	104, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 77, 92, 
	109, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 91, 92, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 79, 92, 111, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 77, 92, 109, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 65, 92, 97, 126, 127, 0, 
	31, 42, 46, 48, 57, 66, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 73, 92, 105, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 78, 92, 110, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 61, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	91, 92, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 79, 92, 111, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 78, 92, 110, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 67, 92, 99, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 69, 92, 101, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 61, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	91, 92, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 81, 92, 113, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 85, 92, 117, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	79, 92, 111, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	80, 92, 112, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 69, 
	92, 101, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 65, 
	92, 97, 126, 127, 0, 31, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 76, 
	92, 108, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 77, 
	92, 109, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 61, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 91, 92, 126, 
	127, 0, 31, 42, 43, 45, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 92, 127, 0, 31, 13, 34, 
	92, 127, 0, 8, 10, 31, 9, 13, 
	32, 44, 13, 0, 9, 11, 127, 9, 
	13, 32, 33, 34, 37, 39, 65, 68, 
	78, 79, 81, 82, 83, 92, 97, 100, 
	110, 111, 113, 114, 115, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 61, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 91, 92, 126, 127, 0, 31, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	13, 34, 58, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 58, 92, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 48, 57, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 93, 127, 
	0, 8, 10, 31, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	46, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 46, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 13, 34, 58, 92, 127, 
	0, 8, 10, 31, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 76, 92, 108, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 71, 92, 103, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 79, 92, 111, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 82, 92, 114, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 73, 92, 105, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 84, 92, 116, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 72, 92, 104, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 77, 92, 109, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 77, 92, 109, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 73, 92, 105, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 67, 92, 99, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 81, 92, 113, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 85, 92, 117, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 76, 92, 108, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 77, 92, 109, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 84, 92, 116, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 76, 92, 108, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 70, 
	84, 91, 92, 102, 116, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 65, 92, 97, 126, 127, 
	0, 31, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 76, 92, 108, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 83, 92, 115, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	69, 92, 101, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	82, 92, 114, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 85, 
	92, 117, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 84, 
	92, 116, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 65, 
	92, 97, 126, 127, 0, 31, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 76, 
	92, 108, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 69, 
	92, 101, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 61, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 70, 84, 91, 
	92, 102, 116, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	76, 92, 108, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 83, 
	92, 115, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 69, 92, 
	101, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 44, 82, 92, 
	114, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 85, 92, 117, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 69, 92, 101, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 65, 92, 97, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	66, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 76, 92, 108, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 77, 92, 109, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 84, 92, 116, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 65, 92, 97, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	66, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 76, 92, 108, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 69, 92, 101, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 70, 84, 91, 92, 102, 
	116, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 65, 
	92, 97, 126, 127, 0, 31, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 76, 92, 
	108, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 83, 92, 115, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 69, 92, 101, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 44, 82, 92, 114, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 85, 92, 117, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 79, 92, 111, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 80, 92, 112, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 61, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 91, 92, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 69, 92, 101, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 65, 92, 97, 126, 127, 
	0, 31, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 76, 92, 108, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 77, 92, 109, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 61, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 91, 92, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 84, 92, 116, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 65, 92, 97, 126, 127, 
	0, 31, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 76, 92, 108, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 69, 92, 101, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 61, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 70, 84, 91, 92, 102, 116, 126, 
	127, 0, 31, 42, 43, 45, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 65, 92, 97, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	66, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 76, 92, 108, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 83, 92, 115, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 69, 92, 101, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 44, 82, 92, 114, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 85, 92, 117, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 80, 92, 112, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 81, 92, 113, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 85, 92, 117, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	79, 92, 111, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	80, 92, 112, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	69, 92, 101, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	65, 92, 97, 126, 127, 0, 31, 42, 
	46, 48, 57, 66, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	76, 92, 108, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	77, 92, 109, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 84, 
	92, 116, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 65, 
	92, 97, 126, 127, 0, 31, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 76, 
	92, 108, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 69, 
	92, 101, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 61, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 70, 84, 91, 
	92, 102, 116, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	76, 92, 108, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 83, 
	92, 115, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 69, 92, 
	101, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 44, 82, 92, 
	114, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 85, 92, 117, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 79, 92, 111, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 78, 92, 110, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 67, 92, 99, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 69, 92, 101, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 80, 92, 112, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 65, 92, 97, 126, 
	127, 0, 31, 42, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 81, 92, 113, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 85, 92, 117, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 69, 92, 101, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 61, 92, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 61, 92, 
	127, 0, 31, 9, 13, 32, 33, 34, 
	37, 39, 91, 92, 126, 127, 0, 31, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 79, 92, 111, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 80, 92, 112, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 61, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 91, 92, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 34, 44, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 69, 92, 101, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 65, 92, 97, 126, 127, 
	0, 31, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 76, 92, 108, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 77, 92, 109, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 61, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 61, 92, 127, 
	0, 31, 9, 13, 32, 33, 34, 37, 
	39, 91, 92, 126, 127, 0, 31, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 84, 92, 116, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 65, 92, 97, 126, 127, 0, 
	31, 42, 46, 48, 57, 66, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 76, 92, 108, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 69, 92, 101, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 61, 92, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 61, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	70, 84, 91, 92, 102, 116, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 65, 92, 97, 126, 
	127, 0, 31, 42, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 76, 92, 108, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 34, 37, 
	39, 44, 83, 92, 115, 126, 127, 0, 
	31, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 34, 37, 39, 
	44, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 34, 44, 92, 127, 0, 
	31, 9, 13, 32, 33, 34, 37, 39, 
	44, 82, 92, 114, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	85, 92, 117, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 79, 
	111, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 78, 110, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 61, 67, 99, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	69, 101, 126, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 126, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 44, 
	61, 9, 13, 32, 33, 34, 37, 39, 
	91, 126, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	37, 39, 44, 61, 80, 112, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 65, 
	97, 126, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 81, 113, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 61, 85, 117, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	69, 101, 126, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 126, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 44, 
	61, 9, 13, 32, 33, 34, 37, 39, 
	91, 126, 42, 43, 45, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	37, 39, 44, 61, 79, 111, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 80, 
	112, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 126, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 44, 61, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	126, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 13, 34, 92, 127, 0, 
	8, 10, 31, 9, 13, 32, 33, 37, 
	39, 44, 61, 69, 101, 126, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 65, 97, 
	126, 42, 46, 48, 57, 66, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 76, 108, 126, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	37, 39, 44, 61, 77, 109, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 44, 61, 9, 13, 32, 
	33, 34, 37, 39, 91, 126, 42, 43, 
	45, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	84, 116, 126, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 65, 97, 126, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 76, 108, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 69, 101, 126, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	37, 39, 44, 61, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	44, 61, 9, 13, 32, 33, 34, 37, 
	39, 70, 84, 91, 102, 116, 126, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	65, 97, 126, 42, 46, 48, 57, 66, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 76, 108, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 83, 115, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 69, 101, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 44, 9, 13, 32, 
	33, 37, 39, 44, 82, 114, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 85, 117, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 79, 111, 126, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	37, 39, 44, 61, 77, 109, 126, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 37, 39, 44, 61, 65, 
	97, 126, 42, 46, 48, 57, 66, 90, 
	95, 122, 9, 13, 32, 33, 37, 39, 
	44, 61, 73, 105, 126, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 37, 39, 44, 61, 78, 110, 126, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 37, 39, 44, 61, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 44, 61, 9, 13, 
	32, 33, 34, 37, 39, 91, 126, 42, 
	43, 45, 46, 48, 57, 65, 90, 95, 
	122, 13, 34, 92, 127, 0, 8, 10, 
	31, 13, 34, 92, 127, 0, 8, 10, 
	31, 13, 34, 0, 9, 11, 127, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 65, 
	68, 78, 79, 81, 82, 83, 92, 97, 
	100, 110, 111, 113, 114, 115, 126, 127, 
	0, 31, 42, 43, 45, 46, 48, 57, 
	66, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	13, 34, 58, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 58, 92, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 46, 92, 127, 
	0, 8, 10, 31, 48, 57, 13, 34, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 127, 0, 
	8, 10, 31, 48, 57, 13, 34, 92, 
	93, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 13, 34, 92, 93, 127, 
	0, 8, 10, 31, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 13, 34, 46, 
	92, 127, 0, 8, 10, 31, 48, 57, 
	13, 34, 46, 92, 127, 0, 8, 10, 
	31, 13, 34, 46, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 13, 34, 46, 
	58, 92, 93, 127, 0, 8, 10, 31, 
	48, 57, 65, 70, 97, 102, 13, 34, 
	46, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 58, 92, 93, 127, 0, 8, 10, 
	31, 13, 34, 92, 127, 0, 8, 10, 
	31, 48, 57, 65, 70, 97, 102, 13, 
	34, 46, 58, 92, 93, 127, 0, 8, 
	10, 31, 48, 57, 65, 70, 97, 102, 
	13, 34, 46, 58, 92, 93, 127, 0, 
	8, 10, 31, 48, 57, 65, 70, 97, 
	102, 13, 34, 46, 58, 92, 93, 127, 
	0, 8, 10, 31, 48, 57, 65, 70, 
	97, 102, 13, 34, 92, 127, 0, 8, 
	10, 31, 48, 57, 13, 34, 46, 92, 
	127, 0, 8, 10, 31, 48, 57, 13, 
	34, 46, 92, 127, 0, 8, 10, 31, 
	48, 57, 13, 34, 46, 92, 127, 0, 
	8, 10, 31, 13, 34, 58, 92, 127, 
	0, 8, 10, 31, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 76, 92, 108, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 71, 92, 103, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 79, 92, 111, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 82, 92, 114, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 73, 92, 105, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 84, 92, 116, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 72, 92, 104, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 77, 92, 109, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 61, 92, 126, 127, 
	0, 31, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 34, 44, 61, 
	92, 127, 0, 31, 9, 13, 32, 33, 
	34, 37, 39, 91, 92, 126, 127, 0, 
	31, 42, 43, 45, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 34, 
	37, 39, 44, 92, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 77, 92, 109, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 65, 92, 97, 126, 127, 0, 31, 
	42, 46, 48, 57, 66, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 73, 92, 105, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 34, 44, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 79, 92, 111, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 78, 92, 110, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 67, 92, 99, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 69, 92, 101, 126, 127, 0, 31, 
	42, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	61, 92, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 34, 44, 61, 92, 127, 0, 31, 
	9, 13, 32, 33, 34, 37, 39, 91, 
	92, 126, 127, 0, 31, 42, 43, 45, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	80, 92, 112, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	65, 92, 97, 126, 127, 0, 31, 42, 
	46, 48, 57, 66, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	81, 92, 113, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	85, 92, 117, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	69, 92, 101, 126, 127, 0, 31, 42, 
	46, 48, 57, 65, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 61, 
	92, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 61, 92, 127, 0, 31, 9, 
	13, 32, 33, 34, 37, 39, 91, 92, 
	126, 127, 0, 31, 42, 43, 45, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 79, 
	92, 111, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 80, 
	92, 112, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 61, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 91, 92, 126, 
	127, 0, 31, 42, 43, 45, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	34, 44, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 69, 
	92, 101, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 65, 
	92, 97, 126, 127, 0, 31, 42, 46, 
	48, 57, 66, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 76, 
	92, 108, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 77, 
	92, 109, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 61, 92, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 61, 92, 127, 0, 31, 9, 13, 
	32, 33, 34, 37, 39, 91, 92, 126, 
	127, 0, 31, 42, 43, 45, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 84, 92, 
	116, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 65, 92, 
	97, 126, 127, 0, 31, 42, 46, 48, 
	57, 66, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 76, 92, 
	108, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 69, 92, 
	101, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 61, 92, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 34, 44, 
	61, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 70, 84, 91, 92, 
	102, 116, 126, 127, 0, 31, 42, 43, 
	45, 46, 48, 57, 65, 90, 95, 122, 
	9, 13, 32, 33, 34, 37, 39, 44, 
	65, 92, 97, 126, 127, 0, 31, 42, 
	46, 48, 57, 66, 90, 95, 122, 9, 
	13, 32, 33, 34, 37, 39, 44, 76, 
	92, 108, 126, 127, 0, 31, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 34, 37, 39, 44, 83, 92, 
	115, 126, 127, 0, 31, 42, 46, 48, 
	57, 65, 90, 95, 122, 9, 13, 32, 
	33, 34, 37, 39, 44, 69, 92, 101, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 34, 
	44, 92, 127, 0, 31, 9, 13, 32, 
	33, 34, 37, 39, 44, 82, 92, 114, 
	126, 127, 0, 31, 42, 46, 48, 57, 
	65, 90, 95, 122, 9, 13, 32, 33, 
	34, 37, 39, 44, 85, 92, 117, 126, 
	127, 0, 31, 42, 46, 48, 57, 65, 
	90, 95, 122, 9, 13, 32, 33, 37, 
	39, 44, 61, 79, 111, 126, 42, 46, 
	48, 57, 65, 90, 95, 122, 9, 13, 
	32, 33, 37, 39, 44, 61, 80, 112, 
	126, 42, 46, 48, 57, 65, 90, 95, 
	122, 9, 13, 32, 33, 37, 39, 44, 
	61, 126, 42, 46, 48, 57, 65, 90, 
	95, 122, 9, 13, 32, 44, 61, 9, 
	13, 32, 33, 34, 37, 39, 91, 126, 
	42, 43, 45, 46, 48, 57, 65, 90, 
	95, 122, 13, 34, 92, 127, 0, 8, 
	10, 31, 13, 34, 92, 127, 0, 8, 
	10, 31, 13, 34, 0, 9, 11, 127, 
	13, 73, 105, 13, 71, 103, 13, 69, 
	101, 13, 83, 115, 13, 84, 116, 9, 
	13, 32, 10, 13, 9, 13, 32, 87, 
	119, 87, 119, 0
};

static const char _thttp_machine_parser_header_WWW_Authenticate_single_lengths[] = {
	0, 4, 2, 2, 2, 2, 1, 2, 
	2, 2, 2, 2, 2, 2, 2, 2, 
	2, 2, 2, 3, 7, 1, 1, 7, 
	3, 3, 3, 3, 3, 21, 9, 5, 
	21, 11, 9, 8, 4, 4, 4, 1, 
	2, 3, 3, 3, 3, 2, 4, 1, 
	2, 1, 2, 1, 2, 2, 2, 2, 
	2, 2, 2, 4, 4, 3, 3, 3, 
	3, 3, 1, 4, 4, 4, 1, 2, 
	2, 2, 2, 11, 11, 11, 11, 11, 
	11, 11, 9, 5, 9, 8, 11, 11, 
	11, 11, 11, 9, 5, 9, 4, 4, 
	2, 2, 7, 24, 12, 8, 11, 11, 
	7, 5, 6, 6, 6, 6, 5, 7, 
	4, 5, 4, 5, 4, 5, 5, 5, 
	5, 5, 5, 5, 7, 7, 6, 6, 
	6, 6, 6, 4, 7, 7, 7, 4, 
	5, 5, 5, 5, 14, 14, 14, 14, 
	14, 14, 14, 14, 12, 8, 11, 11, 
	14, 14, 14, 14, 14, 12, 8, 11, 
	7, 7, 24, 12, 8, 11, 11, 5, 
	6, 6, 6, 6, 5, 7, 4, 5, 
	4, 5, 4, 5, 5, 5, 7, 5, 
	5, 5, 5, 7, 7, 6, 6, 6, 
	6, 6, 4, 7, 7, 7, 4, 5, 
	5, 5, 5, 14, 14, 14, 14, 14, 
	14, 14, 14, 12, 8, 11, 11, 14, 
	14, 14, 14, 14, 12, 8, 11, 14, 
	14, 14, 14, 12, 8, 11, 7, 4, 
	4, 1, 24, 12, 8, 11, 11, 7, 
	5, 6, 6, 6, 6, 5, 7, 4, 
	5, 4, 5, 4, 5, 5, 5, 7, 
	5, 5, 5, 5, 7, 7, 6, 6, 
	6, 6, 6, 4, 7, 7, 7, 4, 
	5, 5, 5, 5, 14, 14, 14, 14, 
	14, 14, 14, 14, 12, 8, 11, 11, 
	14, 14, 14, 14, 14, 12, 8, 11, 
	7, 14, 14, 14, 14, 12, 8, 11, 
	7, 14, 14, 14, 14, 14, 12, 8, 
	11, 7, 4, 4, 1, 7, 24, 12, 
	8, 11, 11, 7, 5, 6, 6, 6, 
	6, 5, 7, 4, 5, 4, 5, 4, 
	5, 5, 5, 7, 5, 5, 5, 5, 
	7, 7, 6, 6, 6, 6, 6, 4, 
	7, 7, 7, 4, 5, 5, 5, 5, 
	14, 14, 14, 14, 14, 14, 14, 14, 
	12, 8, 11, 11, 14, 14, 14, 14, 
	14, 12, 8, 11, 7, 14, 14, 14, 
	14, 12, 8, 11, 7, 14, 14, 14, 
	14, 14, 12, 8, 11, 7, 14, 14, 
	12, 8, 11, 7, 4, 2, 2, 7, 
	24, 12, 8, 11, 11, 5, 6, 6, 
	6, 6, 5, 7, 4, 5, 4, 5, 
	4, 5, 5, 5, 7, 5, 5, 5, 
	5, 7, 7, 6, 6, 6, 6, 6, 
	4, 7, 7, 7, 4, 5, 5, 5, 
	5, 14, 14, 14, 14, 14, 14, 14, 
	14, 12, 8, 11, 11, 14, 14, 14, 
	14, 14, 12, 8, 11, 7, 14, 14, 
	14, 14, 12, 8, 11, 14, 14, 14, 
	14, 14, 12, 8, 11, 14, 14, 12, 
	8, 11, 14, 14, 14, 14, 12, 8, 
	11, 7, 4, 4, 1, 24, 12, 8, 
	11, 11, 7, 5, 6, 6, 6, 6, 
	5, 7, 4, 5, 4, 5, 4, 5, 
	5, 5, 7, 5, 5, 5, 5, 7, 
	7, 6, 6, 6, 6, 6, 4, 7, 
	7, 7, 4, 5, 5, 5, 5, 14, 
	14, 14, 14, 14, 14, 14, 14, 12, 
	8, 11, 11, 14, 14, 14, 14, 14, 
	12, 8, 11, 7, 14, 14, 14, 14, 
	12, 8, 11, 7, 14, 14, 14, 14, 
	14, 12, 8, 11, 7, 14, 14, 12, 
	8, 11, 7, 14, 14, 14, 14, 12, 
	8, 11, 7, 14, 14, 14, 14, 12, 
	8, 15, 13, 13, 13, 13, 7, 13, 
	13, 14, 14, 14, 14, 12, 8, 15, 
	13, 13, 13, 13, 7, 13, 13, 14, 
	14, 14, 14, 12, 8, 11, 7, 14, 
	14, 14, 14, 12, 8, 15, 13, 13, 
	13, 13, 7, 13, 13, 14, 14, 12, 
	8, 11, 7, 14, 14, 14, 14, 12, 
	8, 11, 7, 14, 14, 14, 14, 12, 
	8, 15, 13, 13, 13, 13, 7, 13, 
	13, 14, 14, 14, 14, 14, 12, 8, 
	11, 14, 14, 12, 8, 11, 7, 14, 
	14, 14, 14, 12, 8, 11, 14, 14, 
	14, 14, 12, 8, 15, 13, 13, 13, 
	13, 7, 13, 13, 14, 14, 14, 14, 
	12, 8, 11, 14, 14, 14, 14, 14, 
	12, 8, 11, 14, 14, 12, 8, 11, 
	7, 14, 14, 14, 14, 12, 8, 11, 
	14, 14, 14, 14, 12, 8, 15, 13, 
	13, 13, 13, 7, 13, 13, 11, 11, 
	11, 11, 9, 5, 9, 11, 11, 11, 
	11, 11, 9, 5, 9, 11, 11, 9, 
	5, 9, 4, 11, 11, 11, 11, 9, 
	5, 9, 11, 11, 11, 11, 9, 5, 
	13, 10, 10, 10, 10, 4, 10, 10, 
	11, 11, 11, 11, 11, 9, 5, 9, 
	4, 4, 2, 7, 24, 12, 8, 11, 
	11, 5, 6, 6, 6, 6, 5, 7, 
	4, 5, 4, 5, 4, 5, 5, 5, 
	7, 5, 5, 5, 5, 7, 7, 6, 
	6, 6, 6, 6, 4, 7, 7, 7, 
	4, 5, 5, 5, 5, 14, 14, 14, 
	14, 14, 14, 14, 14, 12, 8, 11, 
	11, 14, 14, 14, 14, 14, 12, 8, 
	11, 7, 14, 14, 14, 14, 12, 8, 
	11, 14, 14, 14, 14, 14, 12, 8, 
	11, 14, 14, 12, 8, 11, 7, 14, 
	14, 14, 14, 12, 8, 11, 14, 14, 
	14, 14, 12, 8, 15, 13, 13, 13, 
	13, 7, 13, 13, 11, 11, 9, 5, 
	9, 4, 4, 2, 3, 3, 3, 3, 
	3, 3, 2, 3, 2, 2, 0
};

static const char _thttp_machine_parser_header_WWW_Authenticate_range_lengths[] = {
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 5, 4, 0, 
	5, 4, 5, 4, 0, 2, 0, 2, 
	3, 3, 3, 3, 0, 3, 3, 1, 
	1, 1, 1, 1, 1, 1, 0, 1, 
	0, 1, 0, 3, 3, 3, 3, 3, 
	3, 0, 3, 3, 3, 3, 1, 1, 
	1, 0, 0, 4, 4, 4, 4, 4, 
	4, 4, 4, 0, 5, 4, 4, 4, 
	4, 4, 4, 4, 0, 5, 2, 2, 
	0, 2, 1, 6, 5, 1, 6, 5, 
	1, 5, 5, 5, 5, 2, 5, 5, 
	3, 3, 3, 3, 3, 3, 3, 2, 
	3, 2, 3, 2, 5, 5, 5, 5, 
	5, 5, 2, 5, 5, 5, 5, 3, 
	3, 3, 2, 2, 5, 5, 5, 5, 
	5, 5, 5, 5, 5, 1, 6, 5, 
	5, 5, 5, 5, 5, 5, 1, 6, 
	1, 1, 6, 5, 1, 6, 5, 5, 
	5, 5, 5, 2, 5, 5, 3, 3, 
	3, 3, 3, 3, 3, 2, 1, 3, 
	2, 3, 2, 5, 5, 5, 5, 5, 
	5, 2, 5, 5, 5, 5, 3, 3, 
	3, 2, 2, 5, 5, 5, 5, 5, 
	5, 5, 5, 5, 1, 6, 5, 5, 
	5, 5, 5, 5, 5, 1, 6, 5, 
	5, 5, 5, 5, 1, 6, 1, 2, 
	0, 2, 6, 5, 1, 6, 5, 1, 
	5, 5, 5, 5, 2, 5, 5, 3, 
	3, 3, 3, 3, 3, 3, 2, 1, 
	3, 2, 3, 2, 5, 5, 5, 5, 
	5, 5, 2, 5, 5, 5, 5, 3, 
	3, 3, 2, 2, 5, 5, 5, 5, 
	5, 5, 5, 5, 5, 1, 6, 5, 
	5, 5, 5, 5, 5, 5, 1, 6, 
	1, 5, 5, 5, 5, 5, 1, 6, 
	1, 5, 5, 5, 5, 5, 5, 1, 
	6, 1, 2, 0, 2, 1, 6, 5, 
	1, 6, 5, 1, 5, 5, 5, 5, 
	2, 5, 5, 3, 3, 3, 3, 3, 
	3, 3, 2, 1, 3, 2, 3, 2, 
	5, 5, 5, 5, 5, 5, 2, 5, 
	5, 5, 5, 3, 3, 3, 2, 2, 
	5, 5, 5, 5, 5, 5, 5, 5, 
	5, 1, 6, 5, 5, 5, 5, 5, 
	5, 5, 1, 6, 1, 5, 5, 5, 
	5, 5, 1, 6, 1, 5, 5, 5, 
	5, 5, 5, 1, 6, 1, 5, 5, 
	5, 1, 6, 1, 2, 0, 2, 1, 
	6, 5, 1, 6, 5, 5, 5, 5, 
	5, 2, 5, 5, 3, 3, 3, 3, 
	3, 3, 3, 2, 1, 3, 2, 3, 
	2, 5, 5, 5, 5, 5, 5, 2, 
	5, 5, 5, 5, 3, 3, 3, 2, 
	2, 5, 5, 5, 5, 5, 5, 5, 
	5, 5, 1, 6, 5, 5, 5, 5, 
	5, 5, 5, 1, 6, 1, 5, 5, 
	5, 5, 5, 1, 6, 5, 5, 5, 
	5, 5, 5, 1, 6, 5, 5, 5, 
	1, 6, 5, 5, 5, 5, 5, 1, 
	6, 1, 2, 0, 2, 6, 5, 1, 
	6, 5, 1, 5, 5, 5, 5, 2, 
	5, 5, 3, 3, 3, 3, 3, 3, 
	3, 2, 1, 3, 2, 3, 2, 5, 
	5, 5, 5, 5, 5, 2, 5, 5, 
	5, 5, 3, 3, 3, 2, 2, 5, 
	5, 5, 5, 5, 5, 5, 5, 5, 
	1, 6, 5, 5, 5, 5, 5, 5, 
	5, 1, 6, 1, 5, 5, 5, 5, 
	5, 1, 6, 1, 5, 5, 5, 5, 
	5, 5, 1, 6, 1, 5, 5, 5, 
	1, 6, 1, 5, 5, 5, 5, 5, 
	1, 6, 1, 5, 5, 5, 5, 5, 
	1, 6, 5, 5, 5, 5, 1, 5, 
	5, 5, 5, 5, 5, 5, 1, 6, 
	5, 5, 5, 5, 1, 5, 5, 5, 
	5, 5, 5, 5, 1, 6, 1, 5, 
	5, 5, 5, 5, 1, 6, 5, 5, 
	5, 5, 1, 5, 5, 5, 5, 5, 
	1, 6, 1, 5, 5, 5, 5, 5, 
	1, 6, 1, 5, 5, 5, 5, 5, 
	1, 6, 5, 5, 5, 5, 1, 5, 
	5, 5, 5, 5, 5, 5, 5, 1, 
	6, 5, 5, 5, 1, 6, 1, 5, 
	5, 5, 5, 5, 1, 6, 5, 5, 
	5, 5, 5, 1, 6, 5, 5, 5, 
	5, 1, 5, 5, 5, 5, 5, 5, 
	5, 1, 6, 5, 5, 5, 5, 5, 
	5, 1, 6, 5, 5, 5, 1, 6, 
	1, 5, 5, 5, 5, 5, 1, 6, 
	5, 5, 5, 5, 5, 1, 6, 5, 
	5, 5, 5, 1, 5, 5, 4, 4, 
	4, 4, 4, 0, 5, 4, 4, 4, 
	4, 4, 4, 0, 5, 4, 4, 4, 
	0, 5, 2, 4, 4, 4, 4, 4, 
	0, 5, 4, 4, 4, 4, 4, 0, 
	5, 4, 4, 4, 4, 0, 4, 4, 
	4, 4, 4, 4, 4, 4, 0, 5, 
	2, 2, 2, 1, 6, 5, 1, 6, 
	5, 5, 5, 5, 5, 2, 5, 5, 
	3, 3, 3, 3, 3, 3, 3, 2, 
	1, 3, 2, 3, 2, 5, 5, 5, 
	5, 5, 5, 2, 5, 5, 5, 5, 
	3, 3, 3, 2, 2, 5, 5, 5, 
	5, 5, 5, 5, 5, 5, 1, 6, 
	5, 5, 5, 5, 5, 5, 5, 1, 
	6, 1, 5, 5, 5, 5, 5, 1, 
	6, 5, 5, 5, 5, 5, 5, 1, 
	6, 5, 5, 5, 1, 6, 1, 5, 
	5, 5, 5, 5, 1, 6, 5, 5, 
	5, 5, 5, 1, 6, 5, 5, 5, 
	5, 1, 5, 5, 4, 4, 4, 0, 
	5, 2, 2, 2, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0
};

static const short _thttp_machine_parser_header_WWW_Authenticate_index_offsets[] = {
	0, 0, 5, 8, 11, 14, 17, 19, 
	22, 25, 28, 31, 34, 37, 40, 43, 
	46, 49, 52, 55, 59, 67, 69, 71, 
	79, 83, 87, 91, 95, 99, 126, 140, 
	146, 173, 189, 204, 217, 222, 229, 234, 
	238, 244, 251, 258, 265, 269, 275, 283, 
	286, 290, 293, 297, 300, 304, 308, 311, 
	315, 318, 322, 325, 333, 341, 348, 355, 
	362, 369, 373, 378, 386, 394, 402, 405, 
	409, 413, 416, 419, 435, 451, 467, 483, 
	499, 515, 531, 545, 551, 566, 579, 595, 
	611, 627, 643, 659, 673, 679, 694, 701, 
	708, 711, 716, 725, 756, 774, 784, 802, 
	819, 828, 839, 851, 863, 875, 884, 895, 
	908, 916, 925, 933, 942, 950, 959, 968, 
	976, 985, 993, 1002, 1010, 1023, 1036, 1048, 
	1060, 1072, 1084, 1093, 1103, 1116, 1129, 1142, 
	1150, 1159, 1168, 1176, 1184, 1204, 1224, 1244, 
	1264, 1284, 1304, 1324, 1344, 1362, 1372, 1390, 
	1407, 1427, 1447, 1467, 1487, 1507, 1525, 1535, 
	1553, 1562, 1571, 1602, 1620, 1630, 1648, 1665, 
	1676, 1688, 1700, 1712, 1721, 1732, 1745, 1753, 
	1762, 1770, 1779, 1787, 1796, 1805, 1813, 1822, 
	1831, 1839, 1848, 1856, 1869, 1882, 1894, 1906, 
	1918, 1930, 1939, 1949, 1962, 1975, 1988, 1996, 
	2005, 2014, 2022, 2030, 2050, 2070, 2090, 2110, 
	2130, 2150, 2170, 2190, 2208, 2218, 2236, 2253, 
	2273, 2293, 2313, 2333, 2353, 2371, 2381, 2399, 
	2419, 2439, 2459, 2479, 2497, 2507, 2525, 2534, 
	2541, 2546, 2550, 2581, 2599, 2609, 2627, 2644, 
	2653, 2664, 2676, 2688, 2700, 2709, 2720, 2733, 
	2741, 2750, 2758, 2767, 2775, 2784, 2793, 2801, 
	2810, 2819, 2827, 2836, 2844, 2857, 2870, 2882, 
	2894, 2906, 2918, 2927, 2937, 2950, 2963, 2976, 
	2984, 2993, 3002, 3010, 3018, 3038, 3058, 3078, 
	3098, 3118, 3138, 3158, 3178, 3196, 3206, 3224, 
	3241, 3261, 3281, 3301, 3321, 3341, 3359, 3369, 
	3387, 3396, 3416, 3436, 3456, 3476, 3494, 3504, 
	3522, 3531, 3551, 3571, 3591, 3611, 3631, 3649, 
	3659, 3677, 3686, 3693, 3698, 3702, 3711, 3742, 
	3760, 3770, 3788, 3805, 3814, 3825, 3837, 3849, 
	3861, 3870, 3881, 3894, 3902, 3911, 3919, 3928, 
	3936, 3945, 3954, 3962, 3971, 3980, 3988, 3997, 
	4005, 4018, 4031, 4043, 4055, 4067, 4079, 4088, 
	4098, 4111, 4124, 4137, 4145, 4154, 4163, 4171, 
	4179, 4199, 4219, 4239, 4259, 4279, 4299, 4319, 
	4339, 4357, 4367, 4385, 4402, 4422, 4442, 4462, 
	4482, 4502, 4520, 4530, 4548, 4557, 4577, 4597, 
	4617, 4637, 4655, 4665, 4683, 4692, 4712, 4732, 
	4752, 4772, 4792, 4810, 4820, 4838, 4847, 4867, 
	4887, 4905, 4915, 4933, 4942, 4949, 4952, 4957, 
	4966, 4997, 5015, 5025, 5043, 5060, 5071, 5083, 
	5095, 5107, 5116, 5127, 5140, 5148, 5157, 5165, 
	5174, 5182, 5191, 5200, 5208, 5217, 5226, 5234, 
	5243, 5251, 5264, 5277, 5289, 5301, 5313, 5325, 
	5334, 5344, 5357, 5370, 5383, 5391, 5400, 5409, 
	5417, 5425, 5445, 5465, 5485, 5505, 5525, 5545, 
	5565, 5585, 5603, 5613, 5631, 5648, 5668, 5688, 
	5708, 5728, 5748, 5766, 5776, 5794, 5803, 5823, 
	5843, 5863, 5883, 5901, 5911, 5929, 5949, 5969, 
	5989, 6009, 6029, 6047, 6057, 6075, 6095, 6115, 
	6133, 6143, 6161, 6181, 6201, 6221, 6241, 6259, 
	6269, 6287, 6296, 6303, 6308, 6312, 6343, 6361, 
	6371, 6389, 6406, 6415, 6426, 6438, 6450, 6462, 
	6471, 6482, 6495, 6503, 6512, 6520, 6529, 6537, 
	6546, 6555, 6563, 6572, 6581, 6589, 6598, 6606, 
	6619, 6632, 6644, 6656, 6668, 6680, 6689, 6699, 
	6712, 6725, 6738, 6746, 6755, 6764, 6772, 6780, 
	6800, 6820, 6840, 6860, 6880, 6900, 6920, 6940, 
	6958, 6968, 6986, 7003, 7023, 7043, 7063, 7083, 
	7103, 7121, 7131, 7149, 7158, 7178, 7198, 7218, 
	7238, 7256, 7266, 7284, 7293, 7313, 7333, 7353, 
	7373, 7393, 7411, 7421, 7439, 7448, 7468, 7488, 
	7506, 7516, 7534, 7543, 7563, 7583, 7603, 7623, 
	7641, 7651, 7669, 7678, 7698, 7718, 7738, 7758, 
	7776, 7786, 7808, 7827, 7846, 7865, 7884, 7893, 
	7912, 7931, 7951, 7971, 7991, 8011, 8029, 8039, 
	8061, 8080, 8099, 8118, 8137, 8146, 8165, 8184, 
	8204, 8224, 8244, 8264, 8282, 8292, 8310, 8319, 
	8339, 8359, 8379, 8399, 8417, 8427, 8449, 8468, 
	8487, 8506, 8525, 8534, 8553, 8572, 8592, 8612, 
	8630, 8640, 8658, 8667, 8687, 8707, 8727, 8747, 
	8765, 8775, 8793, 8802, 8822, 8842, 8862, 8882, 
	8900, 8910, 8932, 8951, 8970, 8989, 9008, 9017, 
	9036, 9055, 9075, 9095, 9115, 9135, 9155, 9173, 
	9183, 9201, 9221, 9241, 9259, 9269, 9287, 9296, 
	9316, 9336, 9356, 9376, 9394, 9404, 9422, 9442, 
	9462, 9482, 9502, 9520, 9530, 9552, 9571, 9590, 
	9609, 9628, 9637, 9656, 9675, 9695, 9715, 9735, 
	9755, 9773, 9783, 9801, 9821, 9841, 9861, 9881, 
	9901, 9919, 9929, 9947, 9967, 9987, 10005, 10015, 
	10033, 10042, 10062, 10082, 10102, 10122, 10140, 10150, 
	10168, 10188, 10208, 10228, 10248, 10266, 10276, 10298, 
	10317, 10336, 10355, 10374, 10383, 10402, 10421, 10437, 
	10453, 10469, 10485, 10499, 10505, 10520, 10536, 10552, 
	10568, 10584, 10600, 10614, 10620, 10635, 10651, 10667, 
	10681, 10687, 10702, 10709, 10725, 10741, 10757, 10773, 
	10787, 10793, 10808, 10824, 10840, 10856, 10872, 10886, 
	10892, 10911, 10926, 10941, 10956, 10971, 10976, 10991, 
	11006, 11022, 11038, 11054, 11070, 11086, 11100, 11106, 
	11121, 11128, 11135, 11140, 11149, 11180, 11198, 11208, 
	11226, 11243, 11254, 11266, 11278, 11290, 11299, 11310, 
	11323, 11331, 11340, 11348, 11357, 11365, 11374, 11383, 
	11391, 11400, 11409, 11417, 11426, 11434, 11447, 11460, 
	11472, 11484, 11496, 11508, 11517, 11527, 11540, 11553, 
	11566, 11574, 11583, 11592, 11600, 11608, 11628, 11648, 
	11668, 11688, 11708, 11728, 11748, 11768, 11786, 11796, 
	11814, 11831, 11851, 11871, 11891, 11911, 11931, 11949, 
	11959, 11977, 11986, 12006, 12026, 12046, 12066, 12084, 
	12094, 12112, 12132, 12152, 12172, 12192, 12212, 12230, 
	12240, 12258, 12278, 12298, 12316, 12326, 12344, 12353, 
	12373, 12393, 12413, 12433, 12451, 12461, 12479, 12499, 
	12519, 12539, 12559, 12577, 12587, 12609, 12628, 12647, 
	12666, 12685, 12694, 12713, 12732, 12748, 12764, 12778, 
	12784, 12799, 12806, 12813, 12818, 12822, 12826, 12830, 
	12834, 12838, 12842, 12845, 12849, 12852, 12855
};

static const short _thttp_machine_parser_header_WWW_Authenticate_indicies[] = {
	0, 2, 0, 2, 1, 3, 3, 1, 
	4, 4, 1, 5, 5, 1, 6, 6, 
	1, 7, 1, 8, 8, 1, 9, 9, 
	1, 10, 10, 1, 11, 11, 1, 12, 
	12, 1, 13, 13, 1, 14, 14, 1, 
	15, 15, 1, 16, 16, 1, 17, 17, 
	1, 18, 18, 1, 19, 19, 1, 19, 
	19, 20, 1, 22, 23, 22, 24, 25, 
	24, 25, 21, 26, 21, 27, 1, 22, 
	26, 22, 24, 25, 24, 25, 21, 26, 
	28, 28, 21, 26, 29, 29, 21, 26, 
	30, 30, 21, 26, 31, 31, 21, 32, 
	26, 32, 21, 33, 26, 33, 34, 34, 
	34, 35, 36, 37, 38, 39, 40, 41, 
	35, 36, 37, 38, 39, 40, 41, 34, 
	34, 34, 34, 34, 34, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 44, 44, 
	44, 44, 44, 21, 47, 26, 47, 48, 
	46, 21, 48, 26, 48, 34, 34, 34, 
	35, 49, 37, 38, 50, 40, 41, 35, 
	49, 37, 38, 50, 40, 41, 34, 34, 
	34, 34, 34, 34, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 51, 51, 44, 
	44, 44, 44, 44, 21, 46, 26, 46, 
	52, 53, 52, 52, 54, 52, 52, 52, 
	52, 52, 52, 21, 55, 43, 55, 52, 
	52, 52, 45, 52, 52, 52, 52, 52, 
	21, 56, 26, 56, 48, 21, 26, 57, 
	58, 21, 21, 21, 53, 55, 43, 55, 
	45, 21, 26, 53, 53, 21, 26, 60, 
	59, 59, 59, 21, 26, 62, 57, 61, 
	61, 61, 21, 26, 62, 57, 63, 63, 
	63, 21, 26, 62, 57, 64, 64, 64, 
	21, 26, 62, 57, 21, 26, 66, 65, 
	59, 59, 21, 26, 67, 62, 57, 68, 
	61, 61, 21, 26, 69, 21, 26, 70, 
	71, 21, 26, 72, 21, 26, 73, 74, 
	21, 26, 75, 21, 26, 57, 76, 21, 
	26, 57, 77, 21, 26, 57, 21, 26, 
	73, 78, 21, 26, 73, 21, 26, 70, 
	79, 21, 26, 70, 21, 26, 67, 62, 
	57, 80, 63, 63, 21, 26, 67, 62, 
	57, 64, 64, 64, 21, 26, 82, 57, 
	81, 81, 81, 21, 26, 84, 57, 83, 
	83, 83, 21, 26, 84, 57, 85, 85, 
	85, 21, 26, 84, 57, 86, 86, 86, 
	21, 26, 84, 57, 21, 26, 87, 81, 
	81, 21, 26, 67, 84, 57, 88, 83, 
	83, 21, 26, 67, 84, 57, 89, 85, 
	85, 21, 26, 67, 84, 57, 86, 86, 
	86, 21, 26, 90, 21, 26, 67, 91, 
	21, 26, 67, 92, 21, 26, 67, 21, 
	26, 66, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 93, 93, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 94, 94, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 95, 95, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 96, 96, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 97, 97, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 98, 98, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 99, 99, 44, 44, 44, 
	44, 44, 21, 100, 43, 100, 44, 44, 
	44, 45, 101, 44, 44, 44, 44, 44, 
	21, 102, 26, 102, 48, 101, 21, 101, 
	26, 101, 103, 53, 103, 103, 54, 103, 
	103, 103, 103, 103, 103, 21, 104, 105, 
	104, 106, 106, 106, 107, 106, 106, 106, 
	106, 106, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 108, 108, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 109, 109, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 110, 110, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 111, 111, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 112, 112, 44, 44, 44, 
	44, 44, 21, 113, 43, 113, 44, 44, 
	44, 45, 114, 44, 44, 44, 44, 44, 
	21, 115, 26, 115, 48, 114, 21, 114, 
	26, 114, 52, 116, 52, 52, 54, 52, 
	52, 52, 52, 52, 52, 21, 26, 119, 
	120, 118, 118, 118, 117, 26, 123, 124, 
	122, 122, 122, 121, 26, 123, 122, 26, 
	125, 121, 121, 122, 126, 26, 126, 57, 
	127, 58, 21, 21, 53, 127, 26, 127, 
	128, 57, 128, 128, 129, 130, 131, 132, 
	133, 134, 135, 58, 129, 130, 131, 132, 
	133, 134, 135, 128, 21, 21, 128, 128, 
	128, 128, 128, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 58, 137, 21, 
	21, 137, 137, 137, 137, 53, 140, 26, 
	140, 57, 127, 139, 58, 21, 21, 53, 
	139, 26, 139, 141, 142, 141, 141, 143, 
	58, 141, 21, 21, 141, 141, 141, 141, 
	141, 53, 144, 43, 144, 141, 57, 141, 
	141, 138, 58, 141, 21, 21, 141, 141, 
	141, 141, 53, 144, 43, 144, 57, 138, 
	58, 21, 21, 53, 26, 57, 146, 58, 
	21, 21, 21, 145, 145, 145, 53, 26, 
	57, 148, 58, 142, 21, 21, 21, 147, 
	147, 147, 53, 26, 57, 148, 58, 142, 
	21, 21, 21, 149, 149, 149, 53, 26, 
	57, 148, 58, 142, 21, 21, 21, 150, 
	150, 150, 53, 26, 57, 148, 58, 142, 
	21, 21, 21, 53, 26, 57, 152, 58, 
	21, 21, 21, 151, 145, 145, 53, 26, 
	57, 153, 148, 58, 142, 21, 21, 21, 
	154, 147, 147, 53, 26, 57, 58, 21, 
	21, 21, 155, 53, 26, 57, 156, 58, 
	21, 21, 21, 157, 53, 26, 57, 58, 
	21, 21, 21, 158, 53, 26, 57, 159, 
	58, 21, 21, 21, 160, 53, 26, 57, 
	58, 21, 21, 21, 161, 53, 26, 57, 
	58, 142, 21, 21, 21, 162, 53, 26, 
	57, 58, 142, 21, 21, 21, 163, 53, 
	26, 57, 58, 142, 21, 21, 21, 53, 
	26, 57, 159, 58, 21, 21, 21, 164, 
	53, 26, 57, 159, 58, 21, 21, 21, 
	53, 26, 57, 156, 58, 21, 21, 21, 
	165, 53, 26, 57, 156, 58, 21, 21, 
	21, 53, 26, 57, 153, 148, 58, 142, 
	21, 21, 21, 166, 149, 149, 53, 26, 
	57, 153, 148, 58, 142, 21, 21, 21, 
	150, 150, 150, 53, 26, 57, 168, 58, 
	142, 21, 21, 21, 167, 167, 167, 53, 
	26, 57, 170, 58, 142, 21, 21, 21, 
	169, 169, 169, 53, 26, 57, 170, 58, 
	142, 21, 21, 21, 171, 171, 171, 53, 
	26, 57, 170, 58, 142, 21, 21, 21, 
	172, 172, 172, 53, 26, 57, 170, 58, 
	142, 21, 21, 21, 53, 26, 57, 58, 
	21, 21, 21, 173, 167, 167, 53, 26, 
	57, 153, 170, 58, 142, 21, 21, 21, 
	174, 169, 169, 53, 26, 57, 153, 170, 
	58, 142, 21, 21, 21, 175, 171, 171, 
	53, 26, 57, 153, 170, 58, 142, 21, 
	21, 21, 172, 172, 172, 53, 26, 57, 
	58, 21, 21, 21, 176, 53, 26, 57, 
	153, 58, 21, 21, 21, 177, 53, 26, 
	57, 153, 58, 21, 21, 21, 178, 53, 
	26, 57, 153, 58, 21, 21, 21, 53, 
	26, 57, 152, 58, 21, 21, 21, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 179, 58, 179, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 180, 58, 180, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 181, 58, 181, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 182, 58, 182, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 183, 58, 183, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 184, 58, 184, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 185, 58, 185, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 186, 58, 186, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	187, 43, 187, 137, 57, 137, 137, 138, 
	188, 58, 137, 21, 21, 137, 137, 137, 
	137, 53, 189, 26, 189, 57, 127, 188, 
	58, 21, 21, 53, 188, 26, 188, 190, 
	142, 190, 190, 143, 58, 190, 21, 21, 
	190, 190, 190, 190, 190, 53, 191, 105, 
	191, 192, 57, 192, 192, 193, 58, 192, 
	21, 21, 192, 192, 192, 192, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	194, 58, 194, 137, 21, 21, 137, 137, 
	137, 137, 53, 136, 43, 136, 137, 57, 
	137, 137, 138, 139, 195, 58, 195, 137, 
	21, 21, 137, 137, 137, 137, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	196, 58, 196, 137, 21, 21, 137, 137, 
	137, 137, 53, 136, 43, 136, 137, 57, 
	137, 137, 138, 139, 197, 58, 197, 137, 
	21, 21, 137, 137, 137, 137, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	198, 58, 198, 137, 21, 21, 137, 137, 
	137, 137, 53, 199, 43, 199, 137, 57, 
	137, 137, 138, 200, 58, 137, 21, 21, 
	137, 137, 137, 137, 53, 201, 26, 201, 
	57, 127, 200, 58, 21, 21, 53, 200, 
	26, 200, 141, 202, 141, 141, 143, 58, 
	141, 21, 21, 141, 141, 141, 141, 141, 
	53, 203, 43, 203, 119, 204, 120, 118, 
	118, 117, 205, 26, 205, 123, 206, 124, 
	122, 122, 121, 206, 26, 206, 207, 123, 
	207, 207, 208, 209, 210, 211, 212, 213, 
	214, 124, 208, 209, 210, 211, 212, 213, 
	214, 207, 122, 122, 207, 207, 207, 207, 
	207, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 124, 216, 122, 122, 216, 
	216, 216, 216, 121, 219, 26, 219, 123, 
	206, 218, 124, 122, 122, 121, 218, 26, 
	218, 220, 125, 220, 220, 221, 124, 220, 
	122, 122, 220, 220, 220, 220, 220, 121, 
	222, 43, 222, 220, 123, 220, 220, 217, 
	124, 220, 122, 122, 220, 220, 220, 220, 
	121, 26, 123, 224, 124, 122, 122, 122, 
	223, 223, 223, 121, 26, 123, 226, 124, 
	227, 122, 122, 122, 225, 225, 225, 121, 
	26, 123, 226, 124, 227, 122, 122, 122, 
	228, 228, 228, 121, 26, 123, 226, 124, 
	227, 122, 122, 122, 229, 229, 229, 121, 
	26, 123, 226, 124, 227, 122, 122, 122, 
	121, 26, 123, 231, 124, 122, 122, 122, 
	230, 223, 223, 121, 26, 123, 232, 226, 
	124, 227, 122, 122, 122, 233, 225, 225, 
	121, 26, 123, 124, 122, 122, 122, 234, 
	121, 26, 123, 235, 124, 122, 122, 122, 
	236, 121, 26, 123, 124, 122, 122, 122, 
	237, 121, 26, 123, 238, 124, 122, 122, 
	122, 239, 121, 26, 123, 124, 122, 122, 
	122, 240, 121, 26, 123, 124, 227, 122, 
	122, 122, 241, 121, 26, 123, 124, 227, 
	122, 122, 122, 242, 121, 26, 123, 124, 
	227, 122, 122, 122, 121, 222, 43, 222, 
	123, 217, 124, 122, 122, 121, 26, 123, 
	238, 124, 122, 122, 122, 243, 121, 26, 
	123, 238, 124, 122, 122, 122, 121, 26, 
	123, 235, 124, 122, 122, 122, 244, 121, 
	26, 123, 235, 124, 122, 122, 122, 121, 
	26, 123, 232, 226, 124, 227, 122, 122, 
	122, 245, 228, 228, 121, 26, 123, 232, 
	226, 124, 227, 122, 122, 122, 229, 229, 
	229, 121, 26, 123, 247, 124, 227, 122, 
	122, 122, 246, 246, 246, 121, 26, 123, 
	249, 124, 227, 122, 122, 122, 248, 248, 
	248, 121, 26, 123, 249, 124, 227, 122, 
	122, 122, 250, 250, 250, 121, 26, 123, 
	249, 124, 227, 122, 122, 122, 251, 251, 
	251, 121, 26, 123, 249, 124, 227, 122, 
	122, 122, 121, 26, 123, 124, 122, 122, 
	122, 252, 246, 246, 121, 26, 123, 232, 
	249, 124, 227, 122, 122, 122, 253, 248, 
	248, 121, 26, 123, 232, 249, 124, 227, 
	122, 122, 122, 254, 250, 250, 121, 26, 
	123, 232, 249, 124, 227, 122, 122, 122, 
	251, 251, 251, 121, 26, 123, 124, 122, 
	122, 122, 255, 121, 26, 123, 232, 124, 
	122, 122, 122, 256, 121, 26, 123, 232, 
	124, 122, 122, 122, 257, 121, 26, 123, 
	232, 124, 122, 122, 122, 121, 26, 123, 
	231, 124, 122, 122, 122, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 258, 
	124, 258, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 259, 124, 259, 216, 122, 
	122, 216, 216, 216, 216, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 260, 
	124, 260, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 261, 124, 261, 216, 122, 
	122, 216, 216, 216, 216, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 262, 
	124, 262, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 263, 124, 263, 216, 122, 
	122, 216, 216, 216, 216, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 264, 
	124, 264, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 265, 124, 265, 216, 122, 
	122, 216, 216, 216, 216, 121, 266, 43, 
	266, 216, 123, 216, 216, 217, 267, 124, 
	216, 122, 122, 216, 216, 216, 216, 121, 
	268, 26, 268, 123, 206, 267, 124, 122, 
	122, 121, 267, 26, 267, 269, 125, 269, 
	269, 221, 124, 269, 122, 122, 269, 269, 
	269, 269, 269, 121, 270, 105, 270, 271, 
	123, 271, 271, 272, 124, 271, 122, 122, 
	271, 271, 271, 271, 121, 215, 43, 215, 
	216, 123, 216, 216, 217, 218, 273, 124, 
	273, 216, 122, 122, 216, 216, 216, 216, 
	121, 215, 43, 215, 216, 123, 216, 216, 
	217, 218, 274, 124, 274, 216, 122, 122, 
	216, 216, 216, 216, 121, 215, 43, 215, 
	216, 123, 216, 216, 217, 218, 275, 124, 
	275, 216, 122, 122, 216, 216, 216, 216, 
	121, 215, 43, 215, 216, 123, 216, 216, 
	217, 218, 276, 124, 276, 216, 122, 122, 
	216, 216, 216, 216, 121, 215, 43, 215, 
	216, 123, 216, 216, 217, 218, 277, 124, 
	277, 216, 122, 122, 216, 216, 216, 216, 
	121, 278, 43, 278, 216, 123, 216, 216, 
	217, 279, 124, 216, 122, 122, 216, 216, 
	216, 216, 121, 280, 26, 280, 123, 206, 
	279, 124, 122, 122, 121, 279, 26, 279, 
	220, 123, 220, 220, 221, 124, 220, 122, 
	122, 220, 220, 220, 220, 220, 121, 215, 
	43, 215, 216, 123, 216, 216, 217, 218, 
	281, 124, 281, 216, 122, 122, 216, 216, 
	216, 216, 121, 215, 43, 215, 216, 123, 
	216, 216, 217, 218, 282, 124, 282, 216, 
	122, 122, 216, 216, 216, 216, 121, 215, 
	43, 215, 216, 123, 216, 216, 217, 218, 
	283, 124, 283, 216, 122, 122, 216, 216, 
	216, 216, 121, 215, 43, 215, 216, 123, 
	216, 216, 217, 218, 284, 124, 284, 216, 
	122, 122, 216, 216, 216, 216, 121, 285, 
	43, 285, 216, 123, 216, 216, 217, 286, 
	124, 216, 122, 122, 216, 216, 216, 216, 
	121, 287, 26, 287, 123, 206, 286, 124, 
	122, 122, 121, 288, 26, 288, 220, 289, 
	220, 220, 221, 124, 220, 122, 122, 220, 
	220, 220, 220, 220, 121, 291, 26, 291, 
	292, 293, 294, 21, 21, 290, 26, 292, 
	294, 21, 21, 21, 290, 295, 296, 295, 
	297, 21, 26, 290, 290, 21, 293, 26, 
	293, 298, 292, 298, 298, 299, 300, 301, 
	302, 303, 304, 305, 294, 299, 300, 301, 
	302, 303, 304, 305, 298, 21, 21, 298, 
	298, 298, 298, 298, 290, 306, 43, 306, 
	307, 292, 307, 307, 308, 309, 294, 307, 
	21, 21, 307, 307, 307, 307, 290, 310, 
	26, 310, 292, 293, 309, 294, 21, 21, 
	290, 309, 26, 309, 311, 312, 311, 311, 
	313, 294, 311, 21, 21, 311, 311, 311, 
	311, 311, 290, 314, 43, 314, 311, 292, 
	311, 311, 308, 294, 311, 21, 21, 311, 
	311, 311, 311, 290, 315, 296, 315, 57, 
	316, 58, 21, 21, 53, 26, 292, 318, 
	294, 21, 21, 21, 317, 317, 317, 290, 
	26, 292, 320, 294, 321, 21, 21, 21, 
	319, 319, 319, 290, 26, 292, 320, 294, 
	321, 21, 21, 21, 322, 322, 322, 290, 
	26, 292, 320, 294, 321, 21, 21, 21, 
	323, 323, 323, 290, 26, 292, 320, 294, 
	321, 21, 21, 21, 290, 26, 292, 325, 
	294, 21, 21, 21, 324, 317, 317, 290, 
	26, 292, 326, 320, 294, 321, 21, 21, 
	21, 327, 319, 319, 290, 26, 292, 294, 
	21, 21, 21, 328, 290, 26, 292, 329, 
	294, 21, 21, 21, 330, 290, 26, 292, 
	294, 21, 21, 21, 331, 290, 26, 292, 
	332, 294, 21, 21, 21, 333, 290, 26, 
	292, 294, 21, 21, 21, 334, 290, 26, 
	292, 294, 321, 21, 21, 21, 335, 290, 
	26, 292, 294, 321, 21, 21, 21, 336, 
	290, 26, 292, 294, 321, 21, 21, 21, 
	290, 314, 43, 314, 292, 308, 294, 21, 
	21, 290, 26, 292, 332, 294, 21, 21, 
	21, 337, 290, 26, 292, 332, 294, 21, 
	21, 21, 290, 26, 292, 329, 294, 21, 
	21, 21, 338, 290, 26, 292, 329, 294, 
	21, 21, 21, 290, 26, 292, 326, 320, 
	294, 321, 21, 21, 21, 339, 322, 322, 
	290, 26, 292, 326, 320, 294, 321, 21, 
	21, 21, 323, 323, 323, 290, 26, 292, 
	341, 294, 321, 21, 21, 21, 340, 340, 
	340, 290, 26, 292, 343, 294, 321, 21, 
	21, 21, 342, 342, 342, 290, 26, 292, 
	343, 294, 321, 21, 21, 21, 344, 344, 
	344, 290, 26, 292, 343, 294, 321, 21, 
	21, 21, 345, 345, 345, 290, 26, 292, 
	343, 294, 321, 21, 21, 21, 290, 26, 
	292, 294, 21, 21, 21, 346, 340, 340, 
	290, 26, 292, 326, 343, 294, 321, 21, 
	21, 21, 347, 342, 342, 290, 26, 292, 
	326, 343, 294, 321, 21, 21, 21, 348, 
	344, 344, 290, 26, 292, 326, 343, 294, 
	321, 21, 21, 21, 345, 345, 345, 290, 
	26, 292, 294, 21, 21, 21, 349, 290, 
	26, 292, 326, 294, 21, 21, 21, 350, 
	290, 26, 292, 326, 294, 21, 21, 21, 
	351, 290, 26, 292, 326, 294, 21, 21, 
	21, 290, 26, 292, 325, 294, 21, 21, 
	21, 290, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 352, 294, 352, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 353, 
	294, 353, 307, 21, 21, 307, 307, 307, 
	307, 290, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 354, 294, 354, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 355, 
	294, 355, 307, 21, 21, 307, 307, 307, 
	307, 290, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 356, 294, 356, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 357, 
	294, 357, 307, 21, 21, 307, 307, 307, 
	307, 290, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 358, 294, 358, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 359, 
	294, 359, 307, 21, 21, 307, 307, 307, 
	307, 290, 360, 43, 360, 307, 292, 307, 
	307, 308, 361, 294, 307, 21, 21, 307, 
	307, 307, 307, 290, 362, 26, 362, 292, 
	293, 361, 294, 21, 21, 290, 361, 26, 
	361, 363, 312, 363, 363, 313, 294, 363, 
	21, 21, 363, 363, 363, 363, 363, 290, 
	364, 105, 364, 365, 292, 365, 365, 366, 
	294, 365, 21, 21, 365, 365, 365, 365, 
	290, 306, 43, 306, 307, 292, 307, 307, 
	308, 309, 367, 294, 367, 307, 21, 21, 
	307, 307, 307, 307, 290, 306, 43, 306, 
	307, 292, 307, 307, 308, 309, 368, 294, 
	368, 307, 21, 21, 307, 307, 307, 307, 
	290, 306, 43, 306, 307, 292, 307, 307, 
	308, 309, 369, 294, 369, 307, 21, 21, 
	307, 307, 307, 307, 290, 306, 43, 306, 
	307, 292, 307, 307, 308, 309, 370, 294, 
	370, 307, 21, 21, 307, 307, 307, 307, 
	290, 306, 43, 306, 307, 292, 307, 307, 
	308, 309, 371, 294, 371, 307, 21, 21, 
	307, 307, 307, 307, 290, 372, 43, 372, 
	307, 292, 307, 307, 308, 373, 294, 307, 
	21, 21, 307, 307, 307, 307, 290, 374, 
	26, 374, 292, 293, 373, 294, 21, 21, 
	290, 373, 26, 373, 311, 375, 311, 311, 
	313, 294, 311, 21, 21, 311, 311, 311, 
	311, 311, 290, 376, 296, 376, 119, 377, 
	120, 118, 118, 117, 306, 43, 306, 307, 
	292, 307, 307, 308, 309, 378, 294, 378, 
	307, 21, 21, 307, 307, 307, 307, 290, 
	306, 43, 306, 307, 292, 307, 307, 308, 
	309, 379, 294, 379, 307, 21, 21, 307, 
	307, 307, 307, 290, 306, 43, 306, 307, 
	292, 307, 307, 308, 309, 380, 294, 380, 
	307, 21, 21, 307, 307, 307, 307, 290, 
	306, 43, 306, 307, 292, 307, 307, 308, 
	309, 381, 294, 381, 307, 21, 21, 307, 
	307, 307, 307, 290, 382, 43, 382, 307, 
	292, 307, 307, 308, 383, 294, 307, 21, 
	21, 307, 307, 307, 307, 290, 384, 26, 
	384, 292, 293, 383, 294, 21, 21, 290, 
	385, 26, 385, 311, 386, 311, 311, 313, 
	294, 311, 21, 21, 311, 311, 311, 311, 
	311, 290, 387, 296, 387, 292, 388, 294, 
	21, 21, 290, 306, 43, 306, 307, 292, 
	307, 307, 308, 309, 389, 294, 389, 307, 
	21, 21, 307, 307, 307, 307, 290, 306, 
	43, 306, 307, 292, 307, 307, 308, 309, 
	390, 294, 390, 307, 21, 21, 307, 307, 
	307, 307, 290, 306, 43, 306, 307, 292, 
	307, 307, 308, 309, 391, 294, 391, 307, 
	21, 21, 307, 307, 307, 307, 290, 306, 
	43, 306, 307, 292, 307, 307, 308, 309, 
	392, 294, 392, 307, 21, 21, 307, 307, 
	307, 307, 290, 306, 43, 306, 307, 292, 
	307, 307, 308, 309, 393, 294, 393, 307, 
	21, 21, 307, 307, 307, 307, 290, 394, 
	43, 394, 307, 292, 307, 307, 308, 395, 
	294, 307, 21, 21, 307, 307, 307, 307, 
	290, 396, 26, 396, 292, 293, 395, 294, 
	21, 21, 290, 397, 26, 397, 311, 398, 
	311, 311, 313, 294, 311, 21, 21, 311, 
	311, 311, 311, 311, 290, 400, 296, 400, 
	401, 402, 403, 21, 21, 399, 26, 401, 
	403, 21, 21, 21, 399, 404, 405, 404, 
	406, 21, 26, 399, 399, 21, 407, 26, 
	407, 401, 408, 403, 21, 21, 399, 408, 
	26, 408, 409, 401, 409, 409, 410, 411, 
	412, 413, 414, 415, 416, 403, 410, 411, 
	412, 413, 414, 415, 416, 409, 21, 21, 
	409, 409, 409, 409, 409, 399, 417, 43, 
	417, 418, 401, 418, 418, 419, 420, 403, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	421, 26, 421, 401, 408, 420, 403, 21, 
	21, 399, 420, 26, 420, 422, 423, 422, 
	422, 424, 403, 422, 21, 21, 422, 422, 
	422, 422, 422, 399, 425, 43, 425, 422, 
	401, 422, 422, 419, 403, 422, 21, 21, 
	422, 422, 422, 422, 399, 426, 405, 426, 
	57, 427, 58, 21, 21, 53, 26, 401, 
	429, 403, 21, 21, 21, 428, 428, 428, 
	399, 26, 401, 431, 403, 432, 21, 21, 
	21, 430, 430, 430, 399, 26, 401, 431, 
	403, 432, 21, 21, 21, 433, 433, 433, 
	399, 26, 401, 431, 403, 432, 21, 21, 
	21, 434, 434, 434, 399, 26, 401, 431, 
	403, 432, 21, 21, 21, 399, 26, 401, 
	436, 403, 21, 21, 21, 435, 428, 428, 
	399, 26, 401, 437, 431, 403, 432, 21, 
	21, 21, 438, 430, 430, 399, 26, 401, 
	403, 21, 21, 21, 439, 399, 26, 401, 
	440, 403, 21, 21, 21, 441, 399, 26, 
	401, 403, 21, 21, 21, 442, 399, 26, 
	401, 443, 403, 21, 21, 21, 444, 399, 
	26, 401, 403, 21, 21, 21, 445, 399, 
	26, 401, 403, 432, 21, 21, 21, 446, 
	399, 26, 401, 403, 432, 21, 21, 21, 
	447, 399, 26, 401, 403, 432, 21, 21, 
	21, 399, 425, 43, 425, 401, 419, 403, 
	21, 21, 399, 26, 401, 443, 403, 21, 
	21, 21, 448, 399, 26, 401, 443, 403, 
	21, 21, 21, 399, 26, 401, 440, 403, 
	21, 21, 21, 449, 399, 26, 401, 440, 
	403, 21, 21, 21, 399, 26, 401, 437, 
	431, 403, 432, 21, 21, 21, 450, 433, 
	433, 399, 26, 401, 437, 431, 403, 432, 
	21, 21, 21, 434, 434, 434, 399, 26, 
	401, 452, 403, 432, 21, 21, 21, 451, 
	451, 451, 399, 26, 401, 454, 403, 432, 
	21, 21, 21, 453, 453, 453, 399, 26, 
	401, 454, 403, 432, 21, 21, 21, 455, 
	455, 455, 399, 26, 401, 454, 403, 432, 
	21, 21, 21, 456, 456, 456, 399, 26, 
	401, 454, 403, 432, 21, 21, 21, 399, 
	26, 401, 403, 21, 21, 21, 457, 451, 
	451, 399, 26, 401, 437, 454, 403, 432, 
	21, 21, 21, 458, 453, 453, 399, 26, 
	401, 437, 454, 403, 432, 21, 21, 21, 
	459, 455, 455, 399, 26, 401, 437, 454, 
	403, 432, 21, 21, 21, 456, 456, 456, 
	399, 26, 401, 403, 21, 21, 21, 460, 
	399, 26, 401, 437, 403, 21, 21, 21, 
	461, 399, 26, 401, 437, 403, 21, 21, 
	21, 462, 399, 26, 401, 437, 403, 21, 
	21, 21, 399, 26, 401, 436, 403, 21, 
	21, 21, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 463, 403, 463, 418, 
	21, 21, 418, 418, 418, 418, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	464, 403, 464, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 465, 403, 465, 418, 
	21, 21, 418, 418, 418, 418, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	466, 403, 466, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 467, 403, 467, 418, 
	21, 21, 418, 418, 418, 418, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	468, 403, 468, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 469, 403, 469, 418, 
	21, 21, 418, 418, 418, 418, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	470, 403, 470, 418, 21, 21, 418, 418, 
	418, 418, 399, 471, 43, 471, 418, 401, 
	418, 418, 419, 472, 403, 418, 21, 21, 
	418, 418, 418, 418, 399, 473, 26, 473, 
	401, 408, 472, 403, 21, 21, 399, 472, 
	26, 472, 474, 423, 474, 474, 424, 403, 
	474, 21, 21, 474, 474, 474, 474, 474, 
	399, 475, 105, 475, 476, 401, 476, 476, 
	477, 403, 476, 21, 21, 476, 476, 476, 
	476, 399, 417, 43, 417, 418, 401, 418, 
	418, 419, 420, 478, 403, 478, 418, 21, 
	21, 418, 418, 418, 418, 399, 417, 43, 
	417, 418, 401, 418, 418, 419, 420, 479, 
	403, 479, 418, 21, 21, 418, 418, 418, 
	418, 399, 417, 43, 417, 418, 401, 418, 
	418, 419, 420, 480, 403, 480, 418, 21, 
	21, 418, 418, 418, 418, 399, 417, 43, 
	417, 418, 401, 418, 418, 419, 420, 481, 
	403, 481, 418, 21, 21, 418, 418, 418, 
	418, 399, 417, 43, 417, 418, 401, 418, 
	418, 419, 420, 482, 403, 482, 418, 21, 
	21, 418, 418, 418, 418, 399, 483, 43, 
	483, 418, 401, 418, 418, 419, 484, 403, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	485, 26, 485, 401, 408, 484, 403, 21, 
	21, 399, 484, 26, 484, 422, 486, 422, 
	422, 424, 403, 422, 21, 21, 422, 422, 
	422, 422, 422, 399, 487, 405, 487, 119, 
	488, 120, 118, 118, 117, 417, 43, 417, 
	418, 401, 418, 418, 419, 420, 489, 403, 
	489, 418, 21, 21, 418, 418, 418, 418, 
	399, 417, 43, 417, 418, 401, 418, 418, 
	419, 420, 490, 403, 490, 418, 21, 21, 
	418, 418, 418, 418, 399, 417, 43, 417, 
	418, 401, 418, 418, 419, 420, 491, 403, 
	491, 418, 21, 21, 418, 418, 418, 418, 
	399, 417, 43, 417, 418, 401, 418, 418, 
	419, 420, 492, 403, 492, 418, 21, 21, 
	418, 418, 418, 418, 399, 493, 43, 493, 
	418, 401, 418, 418, 419, 494, 403, 418, 
	21, 21, 418, 418, 418, 418, 399, 495, 
	26, 495, 401, 408, 494, 403, 21, 21, 
	399, 496, 26, 496, 422, 497, 422, 422, 
	424, 403, 422, 21, 21, 422, 422, 422, 
	422, 422, 399, 498, 405, 498, 292, 499, 
	294, 21, 21, 290, 417, 43, 417, 418, 
	401, 418, 418, 419, 420, 500, 403, 500, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	417, 43, 417, 418, 401, 418, 418, 419, 
	420, 501, 403, 501, 418, 21, 21, 418, 
	418, 418, 418, 399, 417, 43, 417, 418, 
	401, 418, 418, 419, 420, 502, 403, 502, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	417, 43, 417, 418, 401, 418, 418, 419, 
	420, 503, 403, 503, 418, 21, 21, 418, 
	418, 418, 418, 399, 417, 43, 417, 418, 
	401, 418, 418, 419, 420, 504, 403, 504, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	505, 43, 505, 418, 401, 418, 418, 419, 
	506, 403, 418, 21, 21, 418, 418, 418, 
	418, 399, 507, 26, 507, 401, 408, 506, 
	403, 21, 21, 399, 508, 26, 508, 422, 
	509, 422, 422, 424, 403, 422, 21, 21, 
	422, 422, 422, 422, 422, 399, 510, 405, 
	510, 401, 511, 403, 21, 21, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	512, 403, 512, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 513, 403, 513, 418, 
	21, 21, 418, 418, 418, 418, 399, 514, 
	43, 514, 418, 401, 418, 418, 419, 515, 
	403, 418, 21, 21, 418, 418, 418, 418, 
	399, 516, 26, 516, 401, 408, 515, 403, 
	21, 21, 399, 515, 26, 515, 422, 517, 
	422, 422, 424, 403, 422, 21, 21, 422, 
	422, 422, 422, 422, 399, 520, 405, 520, 
	521, 522, 523, 519, 519, 518, 26, 526, 
	527, 525, 525, 525, 524, 26, 526, 525, 
	26, 528, 524, 524, 525, 529, 26, 529, 
	526, 530, 527, 525, 525, 524, 530, 26, 
	530, 531, 526, 531, 531, 532, 533, 534, 
	535, 536, 537, 538, 527, 532, 533, 534, 
	535, 536, 537, 538, 531, 525, 525, 531, 
	531, 531, 531, 531, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 527, 540, 
	525, 525, 540, 540, 540, 540, 524, 543, 
	26, 543, 526, 530, 542, 527, 525, 525, 
	524, 542, 26, 542, 544, 528, 544, 544, 
	545, 527, 544, 525, 525, 544, 544, 544, 
	544, 544, 524, 546, 43, 546, 544, 526, 
	544, 544, 541, 527, 544, 525, 525, 544, 
	544, 544, 544, 524, 26, 526, 548, 527, 
	525, 525, 525, 547, 547, 547, 524, 26, 
	526, 550, 527, 551, 525, 525, 525, 549, 
	549, 549, 524, 26, 526, 550, 527, 551, 
	525, 525, 525, 552, 552, 552, 524, 26, 
	526, 550, 527, 551, 525, 525, 525, 553, 
	553, 553, 524, 26, 526, 550, 527, 551, 
	525, 525, 525, 524, 26, 526, 555, 527, 
	525, 525, 525, 554, 547, 547, 524, 26, 
	526, 556, 550, 527, 551, 525, 525, 525, 
	557, 549, 549, 524, 26, 526, 527, 525, 
	525, 525, 558, 524, 26, 526, 559, 527, 
	525, 525, 525, 560, 524, 26, 526, 527, 
	525, 525, 525, 561, 524, 26, 526, 562, 
	527, 525, 525, 525, 563, 524, 26, 526, 
	527, 525, 525, 525, 564, 524, 26, 526, 
	527, 551, 525, 525, 525, 565, 524, 26, 
	526, 527, 551, 525, 525, 525, 566, 524, 
	26, 526, 527, 551, 525, 525, 525, 524, 
	546, 43, 546, 526, 541, 527, 525, 525, 
	524, 26, 526, 562, 527, 525, 525, 525, 
	567, 524, 26, 526, 562, 527, 525, 525, 
	525, 524, 26, 526, 559, 527, 525, 525, 
	525, 568, 524, 26, 526, 559, 527, 525, 
	525, 525, 524, 26, 526, 556, 550, 527, 
	551, 525, 525, 525, 569, 552, 552, 524, 
	26, 526, 556, 550, 527, 551, 525, 525, 
	525, 553, 553, 553, 524, 26, 526, 571, 
	527, 551, 525, 525, 525, 570, 570, 570, 
	524, 26, 526, 573, 527, 551, 525, 525, 
	525, 572, 572, 572, 524, 26, 526, 573, 
	527, 551, 525, 525, 525, 574, 574, 574, 
	524, 26, 526, 573, 527, 551, 525, 525, 
	525, 575, 575, 575, 524, 26, 526, 573, 
	527, 551, 525, 525, 525, 524, 26, 526, 
	527, 525, 525, 525, 576, 570, 570, 524, 
	26, 526, 556, 573, 527, 551, 525, 525, 
	525, 577, 572, 572, 524, 26, 526, 556, 
	573, 527, 551, 525, 525, 525, 578, 574, 
	574, 524, 26, 526, 556, 573, 527, 551, 
	525, 525, 525, 575, 575, 575, 524, 26, 
	526, 527, 525, 525, 525, 579, 524, 26, 
	526, 556, 527, 525, 525, 525, 580, 524, 
	26, 526, 556, 527, 525, 525, 525, 581, 
	524, 26, 526, 556, 527, 525, 525, 525, 
	524, 26, 526, 555, 527, 525, 525, 525, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 582, 527, 582, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 583, 527, 
	583, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 584, 527, 584, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 585, 527, 
	585, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 586, 527, 586, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 587, 527, 
	587, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 588, 527, 588, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 589, 527, 
	589, 540, 525, 525, 540, 540, 540, 540, 
	524, 590, 43, 590, 540, 526, 540, 540, 
	541, 591, 527, 540, 525, 525, 540, 540, 
	540, 540, 524, 592, 26, 592, 526, 530, 
	591, 527, 525, 525, 524, 591, 26, 591, 
	593, 528, 593, 593, 545, 527, 593, 525, 
	525, 593, 593, 593, 593, 593, 524, 594, 
	105, 594, 595, 526, 595, 595, 596, 527, 
	595, 525, 525, 595, 595, 595, 595, 524, 
	539, 43, 539, 540, 526, 540, 540, 541, 
	542, 597, 527, 597, 540, 525, 525, 540, 
	540, 540, 540, 524, 539, 43, 539, 540, 
	526, 540, 540, 541, 542, 598, 527, 598, 
	540, 525, 525, 540, 540, 540, 540, 524, 
	539, 43, 539, 540, 526, 540, 540, 541, 
	542, 599, 527, 599, 540, 525, 525, 540, 
	540, 540, 540, 524, 539, 43, 539, 540, 
	526, 540, 540, 541, 542, 600, 527, 600, 
	540, 525, 525, 540, 540, 540, 540, 524, 
	539, 43, 539, 540, 526, 540, 540, 541, 
	542, 601, 527, 601, 540, 525, 525, 540, 
	540, 540, 540, 524, 602, 43, 602, 540, 
	526, 540, 540, 541, 603, 527, 540, 525, 
	525, 540, 540, 540, 540, 524, 604, 26, 
	604, 526, 530, 603, 527, 525, 525, 524, 
	603, 26, 603, 544, 605, 544, 544, 545, 
	527, 544, 525, 525, 544, 544, 544, 544, 
	544, 524, 606, 26, 606, 119, 607, 120, 
	118, 118, 117, 539, 43, 539, 540, 526, 
	540, 540, 541, 542, 608, 527, 608, 540, 
	525, 525, 540, 540, 540, 540, 524, 539, 
	43, 539, 540, 526, 540, 540, 541, 542, 
	609, 527, 609, 540, 525, 525, 540, 540, 
	540, 540, 524, 539, 43, 539, 540, 526, 
	540, 540, 541, 542, 610, 527, 610, 540, 
	525, 525, 540, 540, 540, 540, 524, 539, 
	43, 539, 540, 526, 540, 540, 541, 542, 
	611, 527, 611, 540, 525, 525, 540, 540, 
	540, 540, 524, 612, 43, 612, 540, 526, 
	540, 540, 541, 613, 527, 540, 525, 525, 
	540, 540, 540, 540, 524, 614, 26, 614, 
	526, 530, 613, 527, 525, 525, 524, 615, 
	26, 615, 544, 616, 544, 544, 545, 527, 
	544, 525, 525, 544, 544, 544, 544, 544, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 617, 527, 617, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 618, 527, 
	618, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 619, 527, 619, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 620, 527, 
	620, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 621, 527, 621, 540, 525, 525, 
	540, 540, 540, 540, 524, 622, 43, 622, 
	540, 526, 540, 540, 541, 623, 527, 540, 
	525, 525, 540, 540, 540, 540, 524, 624, 
	26, 624, 526, 530, 623, 527, 525, 525, 
	524, 625, 26, 625, 544, 626, 544, 544, 
	545, 527, 544, 525, 525, 544, 544, 544, 
	544, 544, 524, 539, 43, 539, 540, 526, 
	540, 540, 541, 542, 627, 527, 627, 540, 
	525, 525, 540, 540, 540, 540, 524, 539, 
	43, 539, 540, 526, 540, 540, 541, 542, 
	628, 527, 628, 540, 525, 525, 540, 540, 
	540, 540, 524, 629, 43, 629, 540, 526, 
	540, 540, 541, 630, 527, 540, 525, 525, 
	540, 540, 540, 540, 524, 631, 26, 631, 
	526, 530, 630, 527, 525, 525, 524, 630, 
	26, 630, 544, 526, 544, 544, 545, 527, 
	544, 525, 525, 544, 544, 544, 544, 544, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 632, 527, 632, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 633, 527, 
	633, 540, 525, 525, 540, 540, 540, 540, 
	524, 539, 43, 539, 540, 526, 540, 540, 
	541, 542, 634, 527, 634, 540, 525, 525, 
	540, 540, 540, 540, 524, 539, 43, 539, 
	540, 526, 540, 540, 541, 542, 635, 527, 
	635, 540, 525, 525, 540, 540, 540, 540, 
	524, 636, 43, 636, 540, 526, 540, 540, 
	541, 637, 527, 540, 525, 525, 540, 540, 
	540, 540, 524, 638, 26, 638, 526, 530, 
	637, 527, 525, 525, 524, 639, 26, 639, 
	544, 640, 544, 544, 545, 527, 544, 525, 
	525, 544, 544, 544, 544, 544, 524, 642, 
	26, 642, 643, 644, 645, 21, 21, 641, 
	26, 643, 645, 21, 21, 21, 641, 646, 
	647, 646, 648, 21, 26, 641, 641, 21, 
	644, 26, 644, 649, 643, 649, 649, 650, 
	651, 652, 653, 654, 655, 656, 645, 650, 
	651, 652, 653, 654, 655, 656, 649, 21, 
	21, 649, 649, 649, 649, 649, 641, 657, 
	43, 657, 658, 643, 658, 658, 659, 660, 
	645, 658, 21, 21, 658, 658, 658, 658, 
	641, 661, 26, 661, 643, 644, 660, 645, 
	21, 21, 641, 660, 26, 660, 662, 663, 
	662, 662, 664, 645, 662, 21, 21, 662, 
	662, 662, 662, 662, 641, 665, 43, 665, 
	662, 643, 662, 662, 659, 645, 662, 21, 
	21, 662, 662, 662, 662, 641, 666, 647, 
	666, 57, 667, 58, 21, 21, 53, 26, 
	643, 669, 645, 21, 21, 21, 668, 668, 
	668, 641, 26, 643, 671, 645, 672, 21, 
	21, 21, 670, 670, 670, 641, 26, 643, 
	671, 645, 672, 21, 21, 21, 673, 673, 
	673, 641, 26, 643, 671, 645, 672, 21, 
	21, 21, 674, 674, 674, 641, 26, 643, 
	671, 645, 672, 21, 21, 21, 641, 26, 
	643, 676, 645, 21, 21, 21, 675, 668, 
	668, 641, 26, 643, 677, 671, 645, 672, 
	21, 21, 21, 678, 670, 670, 641, 26, 
	643, 645, 21, 21, 21, 679, 641, 26, 
	643, 680, 645, 21, 21, 21, 681, 641, 
	26, 643, 645, 21, 21, 21, 682, 641, 
	26, 643, 683, 645, 21, 21, 21, 684, 
	641, 26, 643, 645, 21, 21, 21, 685, 
	641, 26, 643, 645, 672, 21, 21, 21, 
	686, 641, 26, 643, 645, 672, 21, 21, 
	21, 687, 641, 26, 643, 645, 672, 21, 
	21, 21, 641, 665, 43, 665, 643, 659, 
	645, 21, 21, 641, 26, 643, 683, 645, 
	21, 21, 21, 688, 641, 26, 643, 683, 
	645, 21, 21, 21, 641, 26, 643, 680, 
	645, 21, 21, 21, 689, 641, 26, 643, 
	680, 645, 21, 21, 21, 641, 26, 643, 
	677, 671, 645, 672, 21, 21, 21, 690, 
	673, 673, 641, 26, 643, 677, 671, 645, 
	672, 21, 21, 21, 674, 674, 674, 641, 
	26, 643, 692, 645, 672, 21, 21, 21, 
	691, 691, 691, 641, 26, 643, 694, 645, 
	672, 21, 21, 21, 693, 693, 693, 641, 
	26, 643, 694, 645, 672, 21, 21, 21, 
	695, 695, 695, 641, 26, 643, 694, 645, 
	672, 21, 21, 21, 696, 696, 696, 641, 
	26, 643, 694, 645, 672, 21, 21, 21, 
	641, 26, 643, 645, 21, 21, 21, 697, 
	691, 691, 641, 26, 643, 677, 694, 645, 
	672, 21, 21, 21, 698, 693, 693, 641, 
	26, 643, 677, 694, 645, 672, 21, 21, 
	21, 699, 695, 695, 641, 26, 643, 677, 
	694, 645, 672, 21, 21, 21, 696, 696, 
	696, 641, 26, 643, 645, 21, 21, 21, 
	700, 641, 26, 643, 677, 645, 21, 21, 
	21, 701, 641, 26, 643, 677, 645, 21, 
	21, 21, 702, 641, 26, 643, 677, 645, 
	21, 21, 21, 641, 26, 643, 676, 645, 
	21, 21, 21, 641, 657, 43, 657, 658, 
	643, 658, 658, 659, 660, 703, 645, 703, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	657, 43, 657, 658, 643, 658, 658, 659, 
	660, 704, 645, 704, 658, 21, 21, 658, 
	658, 658, 658, 641, 657, 43, 657, 658, 
	643, 658, 658, 659, 660, 705, 645, 705, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	657, 43, 657, 658, 643, 658, 658, 659, 
	660, 706, 645, 706, 658, 21, 21, 658, 
	658, 658, 658, 641, 657, 43, 657, 658, 
	643, 658, 658, 659, 660, 707, 645, 707, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	657, 43, 657, 658, 643, 658, 658, 659, 
	660, 708, 645, 708, 658, 21, 21, 658, 
	658, 658, 658, 641, 657, 43, 657, 658, 
	643, 658, 658, 659, 660, 709, 645, 709, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	657, 43, 657, 658, 643, 658, 658, 659, 
	660, 710, 645, 710, 658, 21, 21, 658, 
	658, 658, 658, 641, 711, 43, 711, 658, 
	643, 658, 658, 659, 712, 645, 658, 21, 
	21, 658, 658, 658, 658, 641, 713, 26, 
	713, 643, 644, 712, 645, 21, 21, 641, 
	712, 26, 712, 714, 663, 714, 714, 664, 
	645, 714, 21, 21, 714, 714, 714, 714, 
	714, 641, 715, 105, 715, 716, 643, 716, 
	716, 717, 645, 716, 21, 21, 716, 716, 
	716, 716, 641, 657, 43, 657, 658, 643, 
	658, 658, 659, 660, 718, 645, 718, 658, 
	21, 21, 658, 658, 658, 658, 641, 657, 
	43, 657, 658, 643, 658, 658, 659, 660, 
	719, 645, 719, 658, 21, 21, 658, 658, 
	658, 658, 641, 657, 43, 657, 658, 643, 
	658, 658, 659, 660, 720, 645, 720, 658, 
	21, 21, 658, 658, 658, 658, 641, 657, 
	43, 657, 658, 643, 658, 658, 659, 660, 
	721, 645, 721, 658, 21, 21, 658, 658, 
	658, 658, 641, 657, 43, 657, 658, 643, 
	658, 658, 659, 660, 722, 645, 722, 658, 
	21, 21, 658, 658, 658, 658, 641, 723, 
	43, 723, 658, 643, 658, 658, 659, 724, 
	645, 658, 21, 21, 658, 658, 658, 658, 
	641, 725, 26, 725, 643, 644, 724, 645, 
	21, 21, 641, 724, 26, 724, 662, 726, 
	662, 662, 664, 645, 662, 21, 21, 662, 
	662, 662, 662, 662, 641, 727, 647, 727, 
	119, 728, 120, 118, 118, 117, 657, 43, 
	657, 658, 643, 658, 658, 659, 660, 729, 
	645, 729, 658, 21, 21, 658, 658, 658, 
	658, 641, 657, 43, 657, 658, 643, 658, 
	658, 659, 660, 730, 645, 730, 658, 21, 
	21, 658, 658, 658, 658, 641, 657, 43, 
	657, 658, 643, 658, 658, 659, 660, 731, 
	645, 731, 658, 21, 21, 658, 658, 658, 
	658, 641, 657, 43, 657, 658, 643, 658, 
	658, 659, 660, 732, 645, 732, 658, 21, 
	21, 658, 658, 658, 658, 641, 733, 43, 
	733, 658, 643, 658, 658, 659, 734, 645, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	735, 26, 735, 643, 644, 734, 645, 21, 
	21, 641, 736, 26, 736, 662, 737, 662, 
	662, 664, 645, 662, 21, 21, 662, 662, 
	662, 662, 662, 641, 738, 647, 738, 292, 
	739, 294, 21, 21, 290, 657, 43, 657, 
	658, 643, 658, 658, 659, 660, 740, 645, 
	740, 658, 21, 21, 658, 658, 658, 658, 
	641, 657, 43, 657, 658, 643, 658, 658, 
	659, 660, 741, 645, 741, 658, 21, 21, 
	658, 658, 658, 658, 641, 657, 43, 657, 
	658, 643, 658, 658, 659, 660, 742, 645, 
	742, 658, 21, 21, 658, 658, 658, 658, 
	641, 657, 43, 657, 658, 643, 658, 658, 
	659, 660, 743, 645, 743, 658, 21, 21, 
	658, 658, 658, 658, 641, 657, 43, 657, 
	658, 643, 658, 658, 659, 660, 744, 645, 
	744, 658, 21, 21, 658, 658, 658, 658, 
	641, 745, 43, 745, 658, 643, 658, 658, 
	659, 746, 645, 658, 21, 21, 658, 658, 
	658, 658, 641, 747, 26, 747, 643, 644, 
	746, 645, 21, 21, 641, 748, 26, 748, 
	662, 749, 662, 662, 664, 645, 662, 21, 
	21, 662, 662, 662, 662, 662, 641, 750, 
	647, 750, 401, 751, 403, 21, 21, 399, 
	657, 43, 657, 658, 643, 658, 658, 659, 
	660, 752, 645, 752, 658, 21, 21, 658, 
	658, 658, 658, 641, 657, 43, 657, 658, 
	643, 658, 658, 659, 660, 753, 645, 753, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	754, 43, 754, 658, 643, 658, 658, 659, 
	755, 645, 658, 21, 21, 658, 658, 658, 
	658, 641, 756, 26, 756, 643, 644, 755, 
	645, 21, 21, 641, 755, 26, 755, 662, 
	757, 662, 662, 664, 645, 662, 21, 21, 
	662, 662, 662, 662, 662, 641, 758, 647, 
	758, 521, 759, 523, 519, 519, 518, 657, 
	43, 657, 658, 643, 658, 658, 659, 660, 
	760, 645, 760, 658, 21, 21, 658, 658, 
	658, 658, 641, 657, 43, 657, 658, 643, 
	658, 658, 659, 660, 761, 645, 761, 658, 
	21, 21, 658, 658, 658, 658, 641, 657, 
	43, 657, 658, 643, 658, 658, 659, 660, 
	762, 645, 762, 658, 21, 21, 658, 658, 
	658, 658, 641, 657, 43, 657, 658, 643, 
	658, 658, 659, 660, 763, 645, 763, 658, 
	21, 21, 658, 658, 658, 658, 641, 764, 
	43, 764, 658, 643, 658, 658, 659, 765, 
	645, 658, 21, 21, 658, 658, 658, 658, 
	641, 766, 26, 766, 643, 644, 765, 645, 
	21, 21, 641, 767, 26, 767, 662, 768, 
	662, 662, 664, 645, 662, 21, 21, 662, 
	662, 662, 662, 662, 641, 769, 647, 769, 
	643, 770, 645, 21, 21, 641, 657, 43, 
	657, 658, 643, 658, 658, 659, 660, 771, 
	645, 771, 658, 21, 21, 658, 658, 658, 
	658, 641, 657, 43, 657, 658, 643, 658, 
	658, 659, 660, 772, 645, 772, 658, 21, 
	21, 658, 658, 658, 658, 641, 657, 43, 
	657, 658, 643, 658, 658, 659, 660, 773, 
	645, 773, 658, 21, 21, 658, 658, 658, 
	658, 641, 657, 43, 657, 658, 643, 658, 
	658, 659, 660, 774, 645, 774, 658, 21, 
	21, 658, 658, 658, 658, 641, 775, 43, 
	775, 658, 643, 658, 658, 659, 776, 645, 
	658, 21, 21, 658, 658, 658, 658, 641, 
	777, 26, 777, 643, 644, 776, 645, 21, 
	21, 641, 776, 26, 776, 662, 663, 662, 
	662, 778, 779, 664, 645, 778, 779, 662, 
	21, 21, 662, 662, 662, 662, 662, 641, 
	665, 43, 665, 662, 643, 662, 662, 659, 
	780, 645, 780, 662, 21, 21, 662, 662, 
	662, 662, 641, 665, 43, 665, 662, 643, 
	662, 662, 659, 781, 645, 781, 662, 21, 
	21, 662, 662, 662, 662, 641, 665, 43, 
	665, 662, 643, 662, 662, 659, 782, 645, 
	782, 662, 21, 21, 662, 662, 662, 662, 
	641, 665, 43, 665, 662, 643, 662, 662, 
	659, 783, 645, 783, 662, 21, 21, 662, 
	662, 662, 662, 641, 784, 785, 784, 643, 
	786, 645, 21, 21, 641, 665, 43, 665, 
	662, 643, 662, 662, 659, 787, 645, 787, 
	662, 21, 21, 662, 662, 662, 662, 641, 
	665, 43, 665, 662, 643, 662, 662, 659, 
	782, 645, 782, 662, 21, 21, 662, 662, 
	662, 662, 641, 539, 43, 539, 540, 526, 
	540, 540, 541, 542, 788, 527, 788, 540, 
	525, 525, 540, 540, 540, 540, 524, 539, 
	43, 539, 540, 526, 540, 540, 541, 542, 
	789, 527, 789, 540, 525, 525, 540, 540, 
	540, 540, 524, 539, 43, 539, 540, 526, 
	540, 540, 541, 542, 790, 527, 790, 540, 
	525, 525, 540, 540, 540, 540, 524, 539, 
	43, 539, 540, 526, 540, 540, 541, 542, 
	791, 527, 791, 540, 525, 525, 540, 540, 
	540, 540, 524, 792, 43, 792, 540, 526, 
	540, 540, 541, 793, 527, 540, 525, 525, 
	540, 540, 540, 540, 524, 794, 26, 794, 
	526, 530, 793, 527, 525, 525, 524, 793, 
	26, 793, 544, 528, 544, 544, 795, 796, 
	545, 527, 795, 796, 544, 525, 525, 544, 
	544, 544, 544, 544, 524, 546, 43, 546, 
	544, 526, 544, 544, 541, 797, 527, 797, 
	544, 525, 525, 544, 544, 544, 544, 524, 
	546, 43, 546, 544, 526, 544, 544, 541, 
	798, 527, 798, 544, 525, 525, 544, 544, 
	544, 544, 524, 546, 43, 546, 544, 526, 
	544, 544, 541, 799, 527, 799, 544, 525, 
	525, 544, 544, 544, 544, 524, 546, 43, 
	546, 544, 526, 544, 544, 541, 800, 527, 
	800, 544, 525, 525, 544, 544, 544, 544, 
	524, 801, 785, 801, 526, 802, 527, 525, 
	525, 524, 546, 43, 546, 544, 526, 544, 
	544, 541, 803, 527, 803, 544, 525, 525, 
	544, 544, 544, 544, 524, 546, 43, 546, 
	544, 526, 544, 544, 541, 799, 527, 799, 
	544, 525, 525, 544, 544, 544, 544, 524, 
	417, 43, 417, 418, 401, 418, 418, 419, 
	420, 804, 403, 804, 418, 21, 21, 418, 
	418, 418, 418, 399, 417, 43, 417, 418, 
	401, 418, 418, 419, 420, 805, 403, 805, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	417, 43, 417, 418, 401, 418, 418, 419, 
	420, 806, 403, 806, 418, 21, 21, 418, 
	418, 418, 418, 399, 417, 43, 417, 418, 
	401, 418, 418, 419, 420, 807, 403, 807, 
	418, 21, 21, 418, 418, 418, 418, 399, 
	808, 43, 808, 418, 401, 418, 418, 419, 
	809, 403, 418, 21, 21, 418, 418, 418, 
	418, 399, 810, 26, 810, 401, 408, 809, 
	403, 21, 21, 399, 811, 26, 811, 422, 
	812, 422, 422, 424, 403, 422, 21, 21, 
	422, 422, 422, 422, 422, 399, 813, 405, 
	813, 643, 814, 645, 21, 21, 641, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	815, 403, 815, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 816, 403, 816, 418, 
	21, 21, 418, 418, 418, 418, 399, 417, 
	43, 417, 418, 401, 418, 418, 419, 420, 
	817, 403, 817, 418, 21, 21, 418, 418, 
	418, 418, 399, 417, 43, 417, 418, 401, 
	418, 418, 419, 420, 818, 403, 818, 418, 
	21, 21, 418, 418, 418, 418, 399, 819, 
	43, 819, 418, 401, 418, 418, 419, 820, 
	403, 418, 21, 21, 418, 418, 418, 418, 
	399, 821, 26, 821, 401, 408, 820, 403, 
	21, 21, 399, 820, 26, 820, 422, 423, 
	422, 422, 822, 823, 424, 403, 822, 823, 
	422, 21, 21, 422, 422, 422, 422, 422, 
	399, 425, 43, 425, 422, 401, 422, 422, 
	419, 824, 403, 824, 422, 21, 21, 422, 
	422, 422, 422, 399, 425, 43, 425, 422, 
	401, 422, 422, 419, 825, 403, 825, 422, 
	21, 21, 422, 422, 422, 422, 399, 425, 
	43, 425, 422, 401, 422, 422, 419, 826, 
	403, 826, 422, 21, 21, 422, 422, 422, 
	422, 399, 425, 43, 425, 422, 401, 422, 
	422, 419, 827, 403, 827, 422, 21, 21, 
	422, 422, 422, 422, 399, 828, 785, 828, 
	401, 829, 403, 21, 21, 399, 425, 43, 
	425, 422, 401, 422, 422, 419, 830, 403, 
	830, 422, 21, 21, 422, 422, 422, 422, 
	399, 425, 43, 425, 422, 401, 422, 422, 
	419, 826, 403, 826, 422, 21, 21, 422, 
	422, 422, 422, 399, 306, 43, 306, 307, 
	292, 307, 307, 308, 309, 831, 294, 831, 
	307, 21, 21, 307, 307, 307, 307, 290, 
	306, 43, 306, 307, 292, 307, 307, 308, 
	309, 832, 294, 832, 307, 21, 21, 307, 
	307, 307, 307, 290, 833, 43, 833, 307, 
	292, 307, 307, 308, 834, 294, 307, 21, 
	21, 307, 307, 307, 307, 290, 835, 26, 
	835, 292, 293, 834, 294, 21, 21, 290, 
	834, 26, 834, 311, 836, 311, 311, 313, 
	294, 311, 21, 21, 311, 311, 311, 311, 
	311, 290, 837, 296, 837, 521, 838, 523, 
	519, 519, 518, 306, 43, 306, 307, 292, 
	307, 307, 308, 309, 839, 294, 839, 307, 
	21, 21, 307, 307, 307, 307, 290, 306, 
	43, 306, 307, 292, 307, 307, 308, 309, 
	840, 294, 840, 307, 21, 21, 307, 307, 
	307, 307, 290, 306, 43, 306, 307, 292, 
	307, 307, 308, 309, 841, 294, 841, 307, 
	21, 21, 307, 307, 307, 307, 290, 306, 
	43, 306, 307, 292, 307, 307, 308, 309, 
	842, 294, 842, 307, 21, 21, 307, 307, 
	307, 307, 290, 843, 43, 843, 307, 292, 
	307, 307, 308, 844, 294, 307, 21, 21, 
	307, 307, 307, 307, 290, 845, 26, 845, 
	292, 293, 844, 294, 21, 21, 290, 846, 
	26, 846, 311, 847, 311, 311, 313, 294, 
	311, 21, 21, 311, 311, 311, 311, 311, 
	290, 848, 296, 848, 643, 849, 645, 21, 
	21, 641, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 850, 294, 850, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 851, 
	294, 851, 307, 21, 21, 307, 307, 307, 
	307, 290, 306, 43, 306, 307, 292, 307, 
	307, 308, 309, 852, 294, 852, 307, 21, 
	21, 307, 307, 307, 307, 290, 306, 43, 
	306, 307, 292, 307, 307, 308, 309, 853, 
	294, 853, 307, 21, 21, 307, 307, 307, 
	307, 290, 854, 43, 854, 307, 292, 307, 
	307, 308, 855, 294, 307, 21, 21, 307, 
	307, 307, 307, 290, 856, 26, 856, 292, 
	293, 855, 294, 21, 21, 290, 855, 26, 
	855, 311, 312, 311, 311, 857, 858, 313, 
	294, 857, 858, 311, 21, 21, 311, 311, 
	311, 311, 311, 290, 314, 43, 314, 311, 
	292, 311, 311, 308, 859, 294, 859, 311, 
	21, 21, 311, 311, 311, 311, 290, 314, 
	43, 314, 311, 292, 311, 311, 308, 860, 
	294, 860, 311, 21, 21, 311, 311, 311, 
	311, 290, 314, 43, 314, 311, 292, 311, 
	311, 308, 861, 294, 861, 311, 21, 21, 
	311, 311, 311, 311, 290, 314, 43, 314, 
	311, 292, 311, 311, 308, 862, 294, 862, 
	311, 21, 21, 311, 311, 311, 311, 290, 
	863, 785, 863, 292, 864, 294, 21, 21, 
	290, 314, 43, 314, 311, 292, 311, 311, 
	308, 865, 294, 865, 311, 21, 21, 311, 
	311, 311, 311, 290, 314, 43, 314, 311, 
	292, 311, 311, 308, 861, 294, 861, 311, 
	21, 21, 311, 311, 311, 311, 290, 215, 
	43, 215, 216, 123, 216, 216, 217, 218, 
	866, 124, 866, 216, 122, 122, 216, 216, 
	216, 216, 121, 215, 43, 215, 216, 123, 
	216, 216, 217, 218, 867, 124, 867, 216, 
	122, 122, 216, 216, 216, 216, 121, 215, 
	43, 215, 216, 123, 216, 216, 217, 218, 
	868, 124, 868, 216, 122, 122, 216, 216, 
	216, 216, 121, 215, 43, 215, 216, 123, 
	216, 216, 217, 218, 869, 124, 869, 216, 
	122, 122, 216, 216, 216, 216, 121, 215, 
	43, 215, 216, 123, 216, 216, 217, 218, 
	870, 124, 870, 216, 122, 122, 216, 216, 
	216, 216, 121, 871, 43, 871, 216, 123, 
	216, 216, 217, 872, 124, 216, 122, 122, 
	216, 216, 216, 216, 121, 873, 26, 873, 
	123, 206, 872, 124, 122, 122, 121, 874, 
	26, 874, 220, 875, 220, 220, 221, 124, 
	220, 122, 122, 220, 220, 220, 220, 220, 
	121, 215, 43, 215, 216, 123, 216, 216, 
	217, 218, 876, 124, 876, 216, 122, 122, 
	216, 216, 216, 216, 121, 215, 43, 215, 
	216, 123, 216, 216, 217, 218, 877, 124, 
	877, 216, 122, 122, 216, 216, 216, 216, 
	121, 878, 43, 878, 216, 123, 216, 216, 
	217, 879, 124, 216, 122, 122, 216, 216, 
	216, 216, 121, 880, 26, 880, 123, 206, 
	879, 124, 122, 122, 121, 879, 26, 879, 
	220, 881, 220, 220, 221, 124, 220, 122, 
	122, 220, 220, 220, 220, 220, 121, 882, 
	26, 882, 521, 883, 523, 519, 519, 518, 
	215, 43, 215, 216, 123, 216, 216, 217, 
	218, 884, 124, 884, 216, 122, 122, 216, 
	216, 216, 216, 121, 215, 43, 215, 216, 
	123, 216, 216, 217, 218, 885, 124, 885, 
	216, 122, 122, 216, 216, 216, 216, 121, 
	215, 43, 215, 216, 123, 216, 216, 217, 
	218, 886, 124, 886, 216, 122, 122, 216, 
	216, 216, 216, 121, 215, 43, 215, 216, 
	123, 216, 216, 217, 218, 887, 124, 887, 
	216, 122, 122, 216, 216, 216, 216, 121, 
	888, 43, 888, 216, 123, 216, 216, 217, 
	889, 124, 216, 122, 122, 216, 216, 216, 
	216, 121, 890, 26, 890, 123, 206, 889, 
	124, 122, 122, 121, 891, 26, 891, 220, 
	892, 220, 220, 221, 124, 220, 122, 122, 
	220, 220, 220, 220, 220, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 893, 
	124, 893, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 894, 124, 894, 216, 122, 
	122, 216, 216, 216, 216, 121, 215, 43, 
	215, 216, 123, 216, 216, 217, 218, 895, 
	124, 895, 216, 122, 122, 216, 216, 216, 
	216, 121, 215, 43, 215, 216, 123, 216, 
	216, 217, 218, 896, 124, 896, 216, 122, 
	122, 216, 216, 216, 216, 121, 897, 43, 
	897, 216, 123, 216, 216, 217, 898, 124, 
	216, 122, 122, 216, 216, 216, 216, 121, 
	899, 26, 899, 123, 206, 898, 124, 122, 
	122, 121, 898, 26, 898, 220, 125, 220, 
	220, 900, 901, 221, 124, 900, 901, 220, 
	122, 122, 220, 220, 220, 220, 220, 121, 
	222, 43, 222, 220, 123, 220, 220, 217, 
	902, 124, 902, 220, 122, 122, 220, 220, 
	220, 220, 121, 222, 43, 222, 220, 123, 
	220, 220, 217, 903, 124, 903, 220, 122, 
	122, 220, 220, 220, 220, 121, 222, 43, 
	222, 220, 123, 220, 220, 217, 904, 124, 
	904, 220, 122, 122, 220, 220, 220, 220, 
	121, 222, 43, 222, 220, 123, 220, 220, 
	217, 905, 124, 905, 220, 122, 122, 220, 
	220, 220, 220, 121, 906, 785, 906, 123, 
	907, 124, 122, 122, 121, 222, 43, 222, 
	220, 123, 220, 220, 217, 908, 124, 908, 
	220, 122, 122, 220, 220, 220, 220, 121, 
	222, 43, 222, 220, 123, 220, 220, 217, 
	904, 124, 904, 220, 122, 122, 220, 220, 
	220, 220, 121, 136, 43, 136, 137, 57, 
	137, 137, 138, 139, 909, 58, 909, 137, 
	21, 21, 137, 137, 137, 137, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	910, 58, 910, 137, 21, 21, 137, 137, 
	137, 137, 53, 136, 43, 136, 137, 57, 
	137, 137, 138, 139, 911, 58, 911, 137, 
	21, 21, 137, 137, 137, 137, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	912, 58, 912, 137, 21, 21, 137, 137, 
	137, 137, 53, 913, 43, 913, 137, 57, 
	137, 137, 138, 914, 58, 137, 21, 21, 
	137, 137, 137, 137, 53, 915, 26, 915, 
	57, 127, 914, 58, 21, 21, 53, 916, 
	26, 916, 141, 917, 141, 141, 143, 58, 
	141, 21, 21, 141, 141, 141, 141, 141, 
	53, 136, 43, 136, 137, 57, 137, 137, 
	138, 139, 918, 58, 918, 137, 21, 21, 
	137, 137, 137, 137, 53, 136, 43, 136, 
	137, 57, 137, 137, 138, 139, 919, 58, 
	919, 137, 21, 21, 137, 137, 137, 137, 
	53, 136, 43, 136, 137, 57, 137, 137, 
	138, 139, 920, 58, 920, 137, 21, 21, 
	137, 137, 137, 137, 53, 136, 43, 136, 
	137, 57, 137, 137, 138, 139, 921, 58, 
	921, 137, 21, 21, 137, 137, 137, 137, 
	53, 136, 43, 136, 137, 57, 137, 137, 
	138, 139, 922, 58, 922, 137, 21, 21, 
	137, 137, 137, 137, 53, 923, 43, 923, 
	137, 57, 137, 137, 138, 924, 58, 137, 
	21, 21, 137, 137, 137, 137, 53, 925, 
	26, 925, 57, 127, 924, 58, 21, 21, 
	53, 926, 26, 926, 141, 927, 141, 141, 
	143, 58, 141, 21, 21, 141, 141, 141, 
	141, 141, 53, 136, 43, 136, 137, 57, 
	137, 137, 138, 139, 928, 58, 928, 137, 
	21, 21, 137, 137, 137, 137, 53, 136, 
	43, 136, 137, 57, 137, 137, 138, 139, 
	929, 58, 929, 137, 21, 21, 137, 137, 
	137, 137, 53, 930, 43, 930, 137, 57, 
	137, 137, 138, 931, 58, 137, 21, 21, 
	137, 137, 137, 137, 53, 932, 26, 932, 
	57, 127, 931, 58, 21, 21, 53, 931, 
	26, 931, 141, 933, 141, 141, 143, 58, 
	141, 21, 21, 141, 141, 141, 141, 141, 
	53, 934, 43, 934, 521, 935, 523, 519, 
	519, 518, 136, 43, 136, 137, 57, 137, 
	137, 138, 139, 936, 58, 936, 137, 21, 
	21, 137, 137, 137, 137, 53, 136, 43, 
	136, 137, 57, 137, 137, 138, 139, 937, 
	58, 937, 137, 21, 21, 137, 137, 137, 
	137, 53, 136, 43, 136, 137, 57, 137, 
	137, 138, 139, 938, 58, 938, 137, 21, 
	21, 137, 137, 137, 137, 53, 136, 43, 
	136, 137, 57, 137, 137, 138, 139, 939, 
	58, 939, 137, 21, 21, 137, 137, 137, 
	137, 53, 940, 43, 940, 137, 57, 137, 
	137, 138, 941, 58, 137, 21, 21, 137, 
	137, 137, 137, 53, 942, 26, 942, 57, 
	127, 941, 58, 21, 21, 53, 943, 26, 
	943, 141, 944, 141, 141, 143, 58, 141, 
	21, 21, 141, 141, 141, 141, 141, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 945, 58, 945, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 946, 58, 946, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	136, 43, 136, 137, 57, 137, 137, 138, 
	139, 947, 58, 947, 137, 21, 21, 137, 
	137, 137, 137, 53, 136, 43, 136, 137, 
	57, 137, 137, 138, 139, 948, 58, 948, 
	137, 21, 21, 137, 137, 137, 137, 53, 
	949, 43, 949, 137, 57, 137, 137, 138, 
	950, 58, 137, 21, 21, 137, 137, 137, 
	137, 53, 951, 26, 951, 57, 127, 950, 
	58, 21, 21, 53, 950, 26, 950, 141, 
	142, 141, 141, 952, 953, 143, 58, 952, 
	953, 141, 21, 21, 141, 141, 141, 141, 
	141, 53, 144, 43, 144, 141, 57, 141, 
	141, 138, 954, 58, 954, 141, 21, 21, 
	141, 141, 141, 141, 53, 144, 43, 144, 
	141, 57, 141, 141, 138, 955, 58, 955, 
	141, 21, 21, 141, 141, 141, 141, 53, 
	144, 43, 144, 141, 57, 141, 141, 138, 
	956, 58, 956, 141, 21, 21, 141, 141, 
	141, 141, 53, 144, 43, 144, 141, 57, 
	141, 141, 138, 957, 58, 957, 141, 21, 
	21, 141, 141, 141, 141, 53, 958, 785, 
	958, 57, 959, 58, 21, 21, 53, 144, 
	43, 144, 141, 57, 141, 141, 138, 960, 
	58, 960, 141, 21, 21, 141, 141, 141, 
	141, 53, 144, 43, 144, 141, 57, 141, 
	141, 138, 956, 58, 956, 141, 21, 21, 
	141, 141, 141, 141, 53, 42, 43, 42, 
	44, 44, 44, 45, 46, 961, 961, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 962, 962, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 963, 963, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 964, 964, 44, 
	44, 44, 44, 44, 21, 965, 43, 965, 
	44, 44, 44, 45, 966, 44, 44, 44, 
	44, 44, 21, 967, 26, 967, 48, 966, 
	21, 968, 26, 968, 52, 969, 52, 52, 
	54, 52, 52, 52, 52, 52, 52, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	970, 970, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	971, 971, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	972, 972, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	973, 973, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	974, 974, 44, 44, 44, 44, 44, 21, 
	975, 43, 975, 44, 44, 44, 45, 976, 
	44, 44, 44, 44, 44, 21, 977, 26, 
	977, 48, 976, 21, 978, 26, 978, 52, 
	979, 52, 52, 54, 52, 52, 52, 52, 
	52, 52, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 980, 980, 44, 44, 44, 
	44, 44, 21, 42, 43, 42, 44, 44, 
	44, 45, 46, 981, 981, 44, 44, 44, 
	44, 44, 21, 982, 43, 982, 44, 44, 
	44, 45, 983, 44, 44, 44, 44, 44, 
	21, 984, 26, 984, 48, 983, 21, 983, 
	26, 983, 52, 985, 52, 52, 54, 52, 
	52, 52, 52, 52, 52, 21, 26, 521, 
	523, 519, 519, 519, 518, 42, 43, 42, 
	44, 44, 44, 45, 46, 986, 986, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 987, 987, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 988, 988, 44, 
	44, 44, 44, 44, 21, 42, 43, 42, 
	44, 44, 44, 45, 46, 989, 989, 44, 
	44, 44, 44, 44, 21, 990, 43, 990, 
	44, 44, 44, 45, 991, 44, 44, 44, 
	44, 44, 21, 992, 26, 992, 48, 991, 
	21, 993, 26, 993, 52, 994, 52, 52, 
	54, 52, 52, 52, 52, 52, 52, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	995, 995, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	996, 996, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	997, 997, 44, 44, 44, 44, 44, 21, 
	42, 43, 42, 44, 44, 44, 45, 46, 
	998, 998, 44, 44, 44, 44, 44, 21, 
	999, 43, 999, 44, 44, 44, 45, 1000, 
	44, 44, 44, 44, 44, 21, 1001, 26, 
	1001, 48, 1000, 21, 1000, 26, 1000, 52, 
	53, 52, 52, 1002, 1003, 54, 1002, 1003, 
	52, 52, 52, 52, 52, 52, 21, 55, 
	43, 55, 52, 52, 52, 45, 1004, 1004, 
	52, 52, 52, 52, 52, 21, 55, 43, 
	55, 52, 52, 52, 45, 1005, 1005, 52, 
	52, 52, 52, 52, 21, 55, 43, 55, 
	52, 52, 52, 45, 1006, 1006, 52, 52, 
	52, 52, 52, 21, 55, 43, 55, 52, 
	52, 52, 45, 1007, 1007, 52, 52, 52, 
	52, 52, 21, 1008, 785, 1008, 1009, 21, 
	55, 43, 55, 52, 52, 52, 45, 1010, 
	1010, 52, 52, 52, 52, 52, 21, 55, 
	43, 55, 52, 52, 52, 45, 1006, 1006, 
	52, 52, 52, 52, 52, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 1011, 1011, 
	44, 44, 44, 44, 44, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 1012, 1012, 
	44, 44, 44, 44, 44, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 1013, 1013, 
	44, 44, 44, 44, 44, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 1014, 1014, 
	44, 44, 44, 44, 44, 21, 42, 43, 
	42, 44, 44, 44, 45, 46, 1015, 1015, 
	44, 44, 44, 44, 44, 21, 1016, 43, 
	1016, 44, 44, 44, 45, 1017, 44, 44, 
	44, 44, 44, 21, 1018, 26, 1018, 48, 
	1017, 21, 1017, 26, 1017, 52, 1019, 52, 
	52, 54, 52, 52, 52, 52, 52, 52, 
	21, 26, 119, 1021, 118, 118, 118, 1020, 
	26, 123, 1023, 122, 122, 122, 1022, 26, 
	1024, 1022, 1022, 122, 1025, 26, 1025, 57, 
	1026, 58, 21, 21, 53, 1026, 26, 1026, 
	1027, 57, 1027, 1027, 1028, 1029, 1030, 1031, 
	1032, 1033, 1034, 58, 1028, 1029, 1030, 1031, 
	1032, 1033, 1034, 1027, 21, 21, 1027, 1027, 
	1027, 1027, 1027, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 58, 1036, 21, 
	21, 1036, 1036, 1036, 1036, 53, 1039, 26, 
	1039, 57, 1026, 1038, 58, 21, 21, 53, 
	1038, 26, 1038, 1040, 142, 1040, 1040, 1041, 
	58, 1040, 21, 21, 1040, 1040, 1040, 1040, 
	1040, 53, 1042, 43, 1042, 1040, 57, 1040, 
	1040, 1037, 58, 1040, 21, 21, 1040, 1040, 
	1040, 1040, 53, 26, 57, 1044, 58, 21, 
	21, 21, 1043, 1043, 1043, 53, 26, 57, 
	1046, 58, 1047, 21, 21, 21, 1045, 1045, 
	1045, 53, 26, 57, 1046, 58, 1047, 21, 
	21, 21, 1048, 1048, 1048, 53, 26, 57, 
	1046, 58, 1047, 21, 21, 21, 1049, 1049, 
	1049, 53, 26, 57, 1046, 58, 1047, 21, 
	21, 21, 53, 26, 57, 1051, 58, 21, 
	21, 21, 1050, 1043, 1043, 53, 26, 57, 
	1052, 1046, 58, 1047, 21, 21, 21, 1053, 
	1045, 1045, 53, 26, 57, 58, 21, 21, 
	21, 1054, 53, 26, 57, 1055, 58, 21, 
	21, 21, 1056, 53, 26, 57, 58, 21, 
	21, 21, 1057, 53, 26, 57, 1058, 58, 
	21, 21, 21, 1059, 53, 26, 57, 58, 
	21, 21, 21, 1060, 53, 26, 57, 58, 
	1047, 21, 21, 21, 1061, 53, 26, 57, 
	58, 1047, 21, 21, 21, 1062, 53, 26, 
	57, 58, 1047, 21, 21, 21, 53, 1042, 
	43, 1042, 57, 1037, 58, 21, 21, 53, 
	26, 57, 1058, 58, 21, 21, 21, 1063, 
	53, 26, 57, 1058, 58, 21, 21, 21, 
	53, 26, 57, 1055, 58, 21, 21, 21, 
	1064, 53, 26, 57, 1055, 58, 21, 21, 
	21, 53, 26, 57, 1052, 1046, 58, 1047, 
	21, 21, 21, 1065, 1048, 1048, 53, 26, 
	57, 1052, 1046, 58, 1047, 21, 21, 21, 
	1049, 1049, 1049, 53, 26, 57, 1067, 58, 
	1047, 21, 21, 21, 1066, 1066, 1066, 53, 
	26, 57, 1069, 58, 1047, 21, 21, 21, 
	1068, 1068, 1068, 53, 26, 57, 1069, 58, 
	1047, 21, 21, 21, 1070, 1070, 1070, 53, 
	26, 57, 1069, 58, 1047, 21, 21, 21, 
	1071, 1071, 1071, 53, 26, 57, 1069, 58, 
	1047, 21, 21, 21, 53, 26, 57, 58, 
	21, 21, 21, 1072, 1066, 1066, 53, 26, 
	57, 1052, 1069, 58, 1047, 21, 21, 21, 
	1073, 1068, 1068, 53, 26, 57, 1052, 1069, 
	58, 1047, 21, 21, 21, 1074, 1070, 1070, 
	53, 26, 57, 1052, 1069, 58, 1047, 21, 
	21, 21, 1071, 1071, 1071, 53, 26, 57, 
	58, 21, 21, 21, 1075, 53, 26, 57, 
	1052, 58, 21, 21, 21, 1076, 53, 26, 
	57, 1052, 58, 21, 21, 21, 1077, 53, 
	26, 57, 1052, 58, 21, 21, 21, 53, 
	26, 57, 1051, 58, 21, 21, 21, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1078, 58, 1078, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1079, 58, 1079, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1080, 58, 1080, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1081, 58, 1081, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1082, 58, 1082, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1083, 58, 1083, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1084, 58, 1084, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1085, 58, 1085, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1086, 43, 1086, 1036, 57, 1036, 1036, 1037, 
	1087, 58, 1036, 21, 21, 1036, 1036, 1036, 
	1036, 53, 1088, 26, 1088, 57, 1026, 1087, 
	58, 21, 21, 53, 1087, 26, 1087, 1089, 
	142, 1089, 1089, 1041, 58, 1089, 21, 21, 
	1089, 1089, 1089, 1089, 1089, 53, 1090, 105, 
	1090, 1091, 57, 1091, 1091, 1092, 58, 1091, 
	21, 21, 1091, 1091, 1091, 1091, 53, 1035, 
	43, 1035, 1036, 57, 1036, 1036, 1037, 1038, 
	1093, 58, 1093, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1035, 43, 1035, 1036, 57, 
	1036, 1036, 1037, 1038, 1094, 58, 1094, 1036, 
	21, 21, 1036, 1036, 1036, 1036, 53, 1035, 
	43, 1035, 1036, 57, 1036, 1036, 1037, 1038, 
	1095, 58, 1095, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1035, 43, 1035, 1036, 57, 
	1036, 1036, 1037, 1038, 1096, 58, 1096, 1036, 
	21, 21, 1036, 1036, 1036, 1036, 53, 1035, 
	43, 1035, 1036, 57, 1036, 1036, 1037, 1038, 
	1097, 58, 1097, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1098, 43, 1098, 1036, 57, 
	1036, 1036, 1037, 1099, 58, 1036, 21, 21, 
	1036, 1036, 1036, 1036, 53, 1100, 26, 1100, 
	57, 1026, 1099, 58, 21, 21, 53, 1099, 
	26, 1099, 1040, 1101, 1040, 1040, 1041, 58, 
	1040, 21, 21, 1040, 1040, 1040, 1040, 1040, 
	53, 1102, 43, 1102, 119, 1103, 120, 118, 
	118, 117, 1035, 43, 1035, 1036, 57, 1036, 
	1036, 1037, 1038, 1104, 58, 1104, 1036, 21, 
	21, 1036, 1036, 1036, 1036, 53, 1035, 43, 
	1035, 1036, 57, 1036, 1036, 1037, 1038, 1105, 
	58, 1105, 1036, 21, 21, 1036, 1036, 1036, 
	1036, 53, 1035, 43, 1035, 1036, 57, 1036, 
	1036, 1037, 1038, 1106, 58, 1106, 1036, 21, 
	21, 1036, 1036, 1036, 1036, 53, 1035, 43, 
	1035, 1036, 57, 1036, 1036, 1037, 1038, 1107, 
	58, 1107, 1036, 21, 21, 1036, 1036, 1036, 
	1036, 53, 1108, 43, 1108, 1036, 57, 1036, 
	1036, 1037, 1109, 58, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1110, 26, 1110, 57, 
	1026, 1109, 58, 21, 21, 53, 1111, 26, 
	1111, 1040, 917, 1040, 1040, 1041, 58, 1040, 
	21, 21, 1040, 1040, 1040, 1040, 1040, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1112, 58, 1112, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1113, 58, 1113, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1114, 58, 1114, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1035, 43, 1035, 1036, 
	57, 1036, 1036, 1037, 1038, 1115, 58, 1115, 
	1036, 21, 21, 1036, 1036, 1036, 1036, 53, 
	1035, 43, 1035, 1036, 57, 1036, 1036, 1037, 
	1038, 1116, 58, 1116, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1117, 43, 1117, 1036, 
	57, 1036, 1036, 1037, 1118, 58, 1036, 21, 
	21, 1036, 1036, 1036, 1036, 53, 1119, 26, 
	1119, 57, 1026, 1118, 58, 21, 21, 53, 
	1120, 26, 1120, 1040, 927, 1040, 1040, 1041, 
	58, 1040, 21, 21, 1040, 1040, 1040, 1040, 
	1040, 53, 1035, 43, 1035, 1036, 57, 1036, 
	1036, 1037, 1038, 1121, 58, 1121, 1036, 21, 
	21, 1036, 1036, 1036, 1036, 53, 1035, 43, 
	1035, 1036, 57, 1036, 1036, 1037, 1038, 1122, 
	58, 1122, 1036, 21, 21, 1036, 1036, 1036, 
	1036, 53, 1123, 43, 1123, 1036, 57, 1036, 
	1036, 1037, 1124, 58, 1036, 21, 21, 1036, 
	1036, 1036, 1036, 53, 1125, 26, 1125, 57, 
	1026, 1124, 58, 21, 21, 53, 1124, 26, 
	1124, 1040, 1126, 1040, 1040, 1041, 58, 1040, 
	21, 21, 1040, 1040, 1040, 1040, 1040, 53, 
	1127, 43, 1127, 521, 1128, 523, 519, 519, 
	518, 1035, 43, 1035, 1036, 57, 1036, 1036, 
	1037, 1038, 1129, 58, 1129, 1036, 21, 21, 
	1036, 1036, 1036, 1036, 53, 1035, 43, 1035, 
	1036, 57, 1036, 1036, 1037, 1038, 1130, 58, 
	1130, 1036, 21, 21, 1036, 1036, 1036, 1036, 
	53, 1035, 43, 1035, 1036, 57, 1036, 1036, 
	1037, 1038, 1131, 58, 1131, 1036, 21, 21, 
	1036, 1036, 1036, 1036, 53, 1035, 43, 1035, 
	1036, 57, 1036, 1036, 1037, 1038, 1132, 58, 
	1132, 1036, 21, 21, 1036, 1036, 1036, 1036, 
	53, 1133, 43, 1133, 1036, 57, 1036, 1036, 
	1037, 1134, 58, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1135, 26, 1135, 57, 1026, 
	1134, 58, 21, 21, 53, 1136, 26, 1136, 
	1040, 944, 1040, 1040, 1041, 58, 1040, 21, 
	21, 1040, 1040, 1040, 1040, 1040, 53, 1035, 
	43, 1035, 1036, 57, 1036, 1036, 1037, 1038, 
	1137, 58, 1137, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1035, 43, 1035, 1036, 57, 
	1036, 1036, 1037, 1038, 1138, 58, 1138, 1036, 
	21, 21, 1036, 1036, 1036, 1036, 53, 1035, 
	43, 1035, 1036, 57, 1036, 1036, 1037, 1038, 
	1139, 58, 1139, 1036, 21, 21, 1036, 1036, 
	1036, 1036, 53, 1035, 43, 1035, 1036, 57, 
	1036, 1036, 1037, 1038, 1140, 58, 1140, 1036, 
	21, 21, 1036, 1036, 1036, 1036, 53, 1141, 
	43, 1141, 1036, 57, 1036, 1036, 1037, 1142, 
	58, 1036, 21, 21, 1036, 1036, 1036, 1036, 
	53, 1143, 26, 1143, 57, 1026, 1142, 58, 
	21, 21, 53, 1142, 26, 1142, 1040, 142, 
	1040, 1040, 1144, 1145, 1041, 58, 1144, 1145, 
	1040, 21, 21, 1040, 1040, 1040, 1040, 1040, 
	53, 1042, 43, 1042, 1040, 57, 1040, 1040, 
	1037, 1146, 58, 1146, 1040, 21, 21, 1040, 
	1040, 1040, 1040, 53, 1042, 43, 1042, 1040, 
	57, 1040, 1040, 1037, 1147, 58, 1147, 1040, 
	21, 21, 1040, 1040, 1040, 1040, 53, 1042, 
	43, 1042, 1040, 57, 1040, 1040, 1037, 1148, 
	58, 1148, 1040, 21, 21, 1040, 1040, 1040, 
	1040, 53, 1042, 43, 1042, 1040, 57, 1040, 
	1040, 1037, 1149, 58, 1149, 1040, 21, 21, 
	1040, 1040, 1040, 1040, 53, 1150, 785, 1150, 
	57, 1151, 58, 21, 21, 53, 1042, 43, 
	1042, 1040, 57, 1040, 1040, 1037, 1152, 58, 
	1152, 1040, 21, 21, 1040, 1040, 1040, 1040, 
	53, 1042, 43, 1042, 1040, 57, 1040, 1040, 
	1037, 1148, 58, 1148, 1040, 21, 21, 1040, 
	1040, 1040, 1040, 53, 42, 43, 42, 44, 
	44, 44, 45, 46, 1153, 1153, 44, 44, 
	44, 44, 44, 21, 42, 43, 42, 44, 
	44, 44, 45, 46, 1154, 1154, 44, 44, 
	44, 44, 44, 21, 1155, 43, 1155, 44, 
	44, 44, 45, 1156, 44, 44, 44, 44, 
	44, 21, 1157, 26, 1157, 48, 1156, 21, 
	1156, 26, 1156, 52, 1158, 52, 52, 54, 
	52, 52, 52, 52, 52, 52, 21, 26, 
	521, 1160, 519, 519, 519, 1159, 26, 526, 
	1162, 525, 525, 525, 1161, 26, 1163, 1161, 
	1161, 525, 26, 1164, 1164, 21, 26, 1165, 
	1165, 21, 26, 1166, 1166, 21, 26, 1167, 
	1167, 21, 26, 1168, 1168, 21, 1169, 26, 
	1169, 21, 1170, 26, 21, 22, 26, 22, 
	21, 1171, 1171, 1, 6, 6, 1, 1, 
	0
};

static const short _thttp_machine_parser_header_WWW_Authenticate_trans_targs[] = {
	2, 0, 924, 3, 4, 5, 6, 7, 
	8, 9, 10, 11, 12, 13, 14, 15, 
	16, 17, 18, 19, 20, 21, 23, 922, 
	24, 916, 22, 926, 25, 26, 27, 28, 
	29, 29, 30, 33, 792, 750, 757, 908, 
	771, 778, 31, 22, 30, 32, 34, 31, 
	32, 86, 765, 75, 35, 37, 40, 36, 
	36, 38, 39, 41, 74, 42, 45, 43, 
	44, 46, 61, 47, 59, 48, 49, 57, 
	50, 51, 55, 52, 53, 54, 56, 58, 
	60, 62, 70, 63, 66, 64, 65, 67, 
	68, 69, 71, 72, 73, 76, 77, 78, 
	79, 80, 81, 82, 83, 84, 83, 85, 
	36, 22, 85, 32, 87, 88, 89, 90, 
	91, 92, 93, 92, 94, 95, 96, 36, 
	97, 95, 96, 36, 97, 98, 98, 99, 
	100, 140, 152, 708, 715, 723, 729, 736, 
	101, 100, 99, 102, 101, 103, 104, 105, 
	98, 106, 139, 107, 110, 108, 109, 111, 
	126, 112, 124, 113, 114, 122, 115, 116, 
	120, 117, 118, 119, 121, 123, 125, 127, 
	135, 128, 131, 129, 130, 132, 133, 134, 
	136, 137, 138, 141, 142, 143, 144, 145, 
	146, 147, 148, 149, 150, 149, 151, 98, 
	151, 99, 153, 154, 155, 156, 157, 158, 
	159, 158, 160, 161, 162, 161, 162, 163, 
	203, 215, 223, 673, 681, 687, 694, 164, 
	163, 162, 165, 164, 166, 167, 161, 168, 
	202, 169, 172, 182, 170, 171, 173, 189, 
	174, 187, 175, 176, 185, 177, 178, 183, 
	179, 180, 181, 184, 186, 188, 190, 198, 
	191, 194, 192, 193, 195, 196, 197, 199, 
	200, 201, 204, 205, 206, 207, 208, 209, 
	210, 211, 212, 213, 212, 214, 161, 214, 
	162, 216, 217, 218, 219, 220, 221, 222, 
	221, 224, 225, 226, 227, 228, 229, 228, 
	229, 230, 231, 230, 232, 234, 233, 36, 
	22, 32, 235, 276, 288, 297, 305, 645, 
	651, 659, 236, 235, 234, 237, 236, 238, 
	239, 240, 230, 98, 99, 241, 275, 242, 
	245, 255, 243, 244, 246, 262, 247, 260, 
	248, 249, 258, 250, 251, 256, 252, 253, 
	254, 257, 259, 261, 263, 271, 264, 267, 
	265, 266, 268, 269, 270, 272, 273, 274, 
	277, 278, 279, 280, 281, 282, 283, 284, 
	285, 286, 285, 287, 230, 287, 234, 289, 
	290, 291, 292, 293, 294, 295, 294, 296, 
	161, 162, 298, 299, 300, 301, 302, 303, 
	302, 303, 304, 230, 234, 306, 307, 308, 
	309, 310, 311, 312, 311, 312, 313, 314, 
	317, 315, 318, 316, 36, 22, 32, 317, 
	318, 319, 360, 372, 381, 389, 398, 623, 
	631, 320, 319, 318, 321, 320, 322, 323, 
	324, 317, 98, 99, 325, 359, 326, 329, 
	339, 327, 328, 330, 346, 331, 344, 332, 
	333, 342, 334, 335, 340, 336, 337, 338, 
	341, 343, 345, 347, 355, 348, 351, 349, 
	350, 352, 353, 354, 356, 357, 358, 361, 
	362, 363, 364, 365, 366, 367, 368, 369, 
	370, 369, 371, 317, 371, 318, 373, 374, 
	375, 376, 377, 378, 379, 378, 380, 161, 
	162, 382, 383, 384, 385, 386, 387, 386, 
	387, 388, 230, 234, 390, 391, 392, 393, 
	394, 395, 396, 395, 396, 397, 317, 318, 
	399, 400, 401, 402, 401, 403, 404, 405, 
	407, 36, 408, 406, 404, 405, 36, 406, 
	98, 407, 408, 409, 449, 461, 470, 477, 
	485, 490, 609, 410, 409, 408, 411, 410, 
	412, 413, 407, 414, 448, 415, 418, 428, 
	416, 417, 419, 435, 420, 433, 421, 422, 
	431, 423, 424, 429, 425, 426, 427, 430, 
	432, 434, 436, 444, 437, 440, 438, 439, 
	441, 442, 443, 445, 446, 447, 450, 451, 
	452, 453, 454, 455, 456, 457, 458, 459, 
	458, 460, 407, 460, 408, 462, 463, 464, 
	465, 466, 467, 468, 467, 469, 161, 162, 
	471, 472, 473, 474, 475, 476, 475, 476, 
	230, 478, 479, 480, 481, 482, 483, 484, 
	483, 484, 317, 486, 487, 488, 489, 488, 
	491, 492, 493, 494, 495, 496, 495, 496, 
	497, 498, 497, 499, 501, 500, 36, 22, 
	32, 502, 543, 555, 564, 572, 581, 587, 
	595, 503, 502, 501, 504, 503, 505, 506, 
	507, 497, 98, 99, 508, 542, 509, 512, 
	522, 510, 511, 513, 529, 514, 527, 515, 
	516, 525, 517, 518, 523, 519, 520, 521, 
	524, 526, 528, 530, 538, 531, 534, 532, 
	533, 535, 536, 537, 539, 540, 541, 544, 
	545, 546, 547, 548, 549, 550, 551, 552, 
	553, 552, 554, 497, 554, 501, 556, 557, 
	558, 559, 560, 561, 562, 561, 563, 161, 
	162, 565, 566, 567, 568, 569, 570, 569, 
	570, 571, 230, 234, 573, 574, 575, 576, 
	577, 578, 579, 578, 579, 580, 317, 318, 
	582, 583, 584, 585, 584, 586, 407, 408, 
	588, 589, 590, 591, 592, 593, 592, 593, 
	594, 497, 501, 596, 597, 598, 599, 600, 
	601, 600, 602, 607, 603, 604, 605, 606, 
	497, 22, 501, 608, 610, 611, 612, 613, 
	614, 615, 614, 616, 621, 617, 618, 619, 
	620, 407, 408, 622, 624, 625, 626, 627, 
	628, 629, 628, 629, 630, 497, 501, 632, 
	633, 634, 635, 636, 637, 636, 638, 643, 
	639, 640, 641, 642, 317, 318, 644, 646, 
	647, 648, 649, 648, 650, 407, 408, 652, 
	653, 654, 655, 656, 657, 656, 657, 658, 
	497, 501, 660, 661, 662, 663, 664, 665, 
	664, 666, 671, 667, 668, 669, 670, 230, 
	234, 672, 674, 675, 676, 677, 678, 679, 
	680, 679, 680, 317, 682, 683, 684, 685, 
	684, 686, 407, 408, 688, 689, 690, 691, 
	692, 693, 692, 693, 497, 695, 696, 697, 
	698, 699, 700, 699, 701, 706, 702, 703, 
	704, 705, 161, 162, 707, 709, 710, 711, 
	712, 713, 714, 713, 714, 255, 716, 717, 
	718, 719, 720, 721, 722, 721, 722, 339, 
	724, 725, 726, 727, 726, 728, 407, 408, 
	730, 731, 732, 733, 734, 735, 734, 735, 
	522, 737, 738, 739, 740, 741, 742, 741, 
	743, 748, 744, 745, 746, 747, 98, 99, 
	749, 751, 752, 753, 754, 755, 756, 755, 
	756, 231, 758, 759, 760, 761, 762, 763, 
	764, 763, 764, 314, 766, 767, 768, 769, 
	768, 770, 772, 773, 774, 775, 776, 777, 
	776, 777, 498, 779, 780, 781, 782, 783, 
	784, 783, 785, 790, 786, 787, 788, 789, 
	36, 32, 791, 793, 794, 795, 796, 797, 
	798, 799, 798, 800, 801, 802, 801, 802, 
	803, 803, 804, 805, 845, 857, 866, 873, 
	881, 887, 894, 806, 805, 804, 807, 806, 
	808, 809, 803, 810, 844, 811, 814, 824, 
	812, 813, 815, 831, 816, 829, 817, 818, 
	827, 819, 820, 825, 821, 822, 823, 826, 
	828, 830, 832, 840, 833, 836, 834, 835, 
	837, 838, 839, 841, 842, 843, 846, 847, 
	848, 849, 850, 851, 852, 853, 854, 855, 
	854, 856, 803, 856, 804, 858, 859, 860, 
	861, 862, 863, 864, 863, 865, 161, 162, 
	867, 868, 869, 870, 871, 872, 871, 872, 
	874, 875, 876, 877, 878, 879, 880, 879, 
	880, 882, 883, 884, 885, 884, 886, 407, 
	408, 888, 889, 890, 891, 892, 893, 892, 
	893, 895, 896, 897, 898, 899, 900, 899, 
	901, 906, 902, 903, 904, 905, 803, 804, 
	907, 909, 910, 911, 912, 911, 913, 914, 
	915, 914, 915, 803, 917, 918, 919, 920, 
	921, 29, 923, 925
};

static const char _thttp_machine_parser_header_WWW_Authenticate_trans_actions[] = {
	9, 0, 7, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 27, 0, 0, 0, 0, 
	5, 0, 1, 1, 1, 1, 1, 1, 
	1, 1, 25, 25, 0, 25, 0, 0, 
	0, 1, 1, 0, 0, 0, 0, 25, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	21, 21, 0, 21, 0, 0, 0, 0, 
	0, 25, 0, 0, 0, 1, 1, 29, 
	1, 0, 0, 13, 0, 13, 0, 0, 
	1, 1, 1, 1, 1, 1, 1, 1, 
	25, 0, 25, 0, 0, 0, 0, 0, 
	25, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 25, 0, 0, 1, 21, 
	0, 21, 0, 0, 0, 0, 0, 25, 
	0, 0, 0, 41, 41, 0, 0, 1, 
	1, 1, 1, 1, 1, 1, 1, 25, 
	0, 25, 0, 0, 0, 0, 25, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 25, 0, 0, 1, 21, 0, 
	21, 0, 0, 0, 0, 0, 25, 0, 
	0, 0, 0, 0, 0, 25, 0, 0, 
	1, 47, 0, 0, 0, 0, 0, 15, 
	15, 15, 1, 1, 1, 1, 1, 1, 
	1, 1, 25, 0, 25, 0, 0, 0, 
	0, 0, 25, 15, 15, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	25, 0, 0, 1, 21, 0, 21, 0, 
	0, 0, 0, 0, 25, 0, 0, 0, 
	32, 32, 0, 0, 0, 0, 25, 0, 
	0, 1, 1, 15, 15, 0, 0, 0, 
	0, 0, 25, 0, 0, 1, 1, 0, 
	15, 0, 15, 0, 17, 17, 17, 0, 
	0, 1, 1, 1, 1, 1, 1, 1, 
	1, 25, 0, 25, 0, 0, 0, 0, 
	0, 25, 17, 17, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 25, 
	0, 0, 1, 21, 0, 21, 0, 0, 
	0, 0, 0, 25, 0, 0, 0, 35, 
	35, 0, 0, 0, 0, 25, 0, 0, 
	1, 1, 17, 17, 0, 0, 0, 0, 
	0, 25, 0, 0, 1, 1, 17, 17, 
	0, 0, 25, 0, 0, 0, 1, 1, 
	53, 38, 53, 1, 0, 0, 23, 0, 
	23, 0, 0, 1, 1, 1, 1, 1, 
	1, 1, 1, 25, 0, 25, 0, 0, 
	0, 0, 25, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 25, 0, 
	0, 1, 21, 0, 21, 0, 0, 0, 
	0, 0, 25, 0, 0, 23, 1, 1, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	38, 0, 0, 0, 0, 0, 25, 0, 
	0, 1, 38, 0, 0, 25, 0, 0, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	38, 0, 0, 0, 0, 0, 11, 11, 
	11, 1, 1, 1, 1, 1, 1, 1, 
	1, 25, 0, 25, 0, 0, 0, 0, 
	0, 25, 11, 11, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 25, 
	0, 0, 1, 21, 0, 21, 0, 0, 
	0, 0, 0, 25, 0, 0, 0, 44, 
	44, 0, 0, 0, 0, 25, 0, 0, 
	1, 1, 11, 11, 0, 0, 0, 0, 
	0, 25, 0, 0, 1, 1, 11, 11, 
	0, 0, 25, 0, 0, 0, 44, 44, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	1, 11, 11, 0, 0, 0, 0, 25, 
	0, 0, 1, 1, 0, 0, 0, 0, 
	19, 19, 19, 0, 0, 0, 0, 0, 
	25, 0, 0, 1, 1, 0, 0, 0, 
	0, 19, 19, 0, 0, 0, 0, 0, 
	25, 0, 0, 1, 1, 17, 17, 0, 
	0, 0, 0, 25, 0, 0, 1, 1, 
	0, 0, 0, 0, 19, 19, 0, 0, 
	0, 25, 0, 0, 0, 50, 50, 0, 
	0, 0, 0, 25, 0, 0, 1, 1, 
	15, 15, 0, 0, 0, 0, 25, 0, 
	0, 1, 1, 0, 0, 0, 0, 19, 
	19, 0, 0, 0, 0, 0, 0, 25, 
	0, 0, 1, 47, 0, 0, 25, 0, 
	0, 13, 1, 1, 0, 0, 0, 0, 
	25, 0, 0, 1, 29, 0, 0, 0, 
	0, 25, 0, 0, 1, 1, 0, 0, 
	0, 0, 19, 19, 0, 0, 0, 0, 
	0, 25, 0, 0, 1, 1, 0, 0, 
	0, 0, 0, 25, 0, 0, 1, 1, 
	0, 0, 25, 0, 0, 0, 41, 41, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	1, 0, 0, 0, 0, 25, 0, 0, 
	1, 1, 0, 0, 0, 0, 19, 19, 
	0, 0, 0, 0, 0, 25, 0, 0, 
	1, 1, 0, 0, 0, 0, 0, 25, 
	0, 0, 1, 1, 0, 0, 25, 0, 
	0, 0, 0, 0, 0, 0, 25, 0, 
	0, 1, 1, 0, 0, 0, 0, 25, 
	0, 0, 1, 1, 0, 0, 0, 0, 
	19, 19, 0, 0, 0, 0, 0, 0, 
	25, 0, 0, 0, 1, 1, 0, 0, 
	13, 0, 0, 1, 1, 1, 1, 1, 
	1, 1, 1, 25, 0, 25, 0, 0, 
	0, 0, 25, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 0, 25, 0, 
	0, 1, 21, 0, 21, 0, 0, 0, 
	0, 0, 25, 0, 0, 0, 56, 56, 
	0, 0, 0, 0, 25, 0, 0, 1, 
	0, 0, 0, 0, 0, 25, 0, 0, 
	1, 0, 0, 25, 0, 0, 0, 56, 
	56, 0, 0, 0, 0, 25, 0, 0, 
	1, 0, 0, 0, 0, 25, 0, 0, 
	1, 1, 0, 0, 0, 0, 19, 19, 
	0, 0, 0, 25, 0, 0, 0, 1, 
	1, 0, 0, 23, 0, 0, 0, 0, 
	0, 3, 0, 0
};

static const int thttp_machine_parser_header_WWW_Authenticate_start = 1;
static const int thttp_machine_parser_header_WWW_Authenticate_first_final = 926;
static const int thttp_machine_parser_header_WWW_Authenticate_error = 0;

static const int thttp_machine_parser_header_WWW_Authenticate_en_main = 1;


/* #line 186 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	
/* #line 4441 "./src/headers/thttp_header_WWW_Authenticate.c" */
	{
	cs = thttp_machine_parser_header_WWW_Authenticate_start;
	}

/* #line 187 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	
/* #line 4448 "./src/headers/thttp_header_WWW_Authenticate.c" */
	{
	int _klen;
	unsigned int _trans;
	const char *_acts;
	unsigned int _nacts;
	const char *_keys;

	if ( p == pe )
		goto _test_eof;
	if ( cs == 0 )
		goto _out;
_resume:
	_keys = _thttp_machine_parser_header_WWW_Authenticate_trans_keys + _thttp_machine_parser_header_WWW_Authenticate_key_offsets[cs];
	_trans = _thttp_machine_parser_header_WWW_Authenticate_index_offsets[cs];

	_klen = _thttp_machine_parser_header_WWW_Authenticate_single_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + _klen - 1;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + ((_upper-_lower) >> 1);
			if ( (*p) < *_mid )
				_upper = _mid - 1;
			else if ( (*p) > *_mid )
				_lower = _mid + 1;
			else {
				_trans += (_mid - _keys);
				goto _match;
			}
		}
		_keys += _klen;
		_trans += _klen;
	}

	_klen = _thttp_machine_parser_header_WWW_Authenticate_range_lengths[cs];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + (_klen<<1) - 2;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + (((_upper-_lower) >> 1) & ~1);
			if ( (*p) < _mid[0] )
				_upper = _mid - 2;
			else if ( (*p) > _mid[1] )
				_lower = _mid + 2;
			else {
				_trans += ((_mid - _keys)>>1);
				goto _match;
			}
		}
		_trans += _klen;
	}

_match:
	_trans = _thttp_machine_parser_header_WWW_Authenticate_indicies[_trans];
	cs = _thttp_machine_parser_header_WWW_Authenticate_trans_targs[_trans];

	if ( _thttp_machine_parser_header_WWW_Authenticate_trans_actions[_trans] == 0 )
		goto _again;

	_acts = _thttp_machine_parser_header_WWW_Authenticate_actions + _thttp_machine_parser_header_WWW_Authenticate_trans_actions[_trans];
	_nacts = (unsigned int) *_acts++;
	while ( _nacts-- > 0 )
	{
		switch ( *_acts++ )
		{
	case 0:
/* #line 50 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		tag_start = p;
	}
	break;
	case 1:
/* #line 54 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		hdr_WWW_Authenticate->scheme = tsk_strdup("Digest");
	}
	break;
	case 2:
/* #line 58 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		hdr_WWW_Authenticate->scheme = tsk_strdup("Basic");
	}
	break;
	case 3:
/* #line 62 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		THTTP_HEADER(hdr_WWW_Authenticate)->type = thttp_htype_WWW_Authenticate;
	}
	break;
	case 4:
/* #line 66 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		THTTP_HEADER(hdr_WWW_Authenticate)->type = thttp_htype_Proxy_Authenticate;
	}
	break;
	case 5:
/* #line 70 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->realm);
		tsk_strunquote(&hdr_WWW_Authenticate->realm);
	}
	break;
	case 6:
/* #line 75 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->domain);
		//tsk_strunquote(&hdr_WWW_Authenticate->domain);
	}
	break;
	case 7:
/* #line 80 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->nonce);
		tsk_strunquote(&hdr_WWW_Authenticate->nonce);
	}
	break;
	case 8:
/* #line 85 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->opaque);
		tsk_strunquote(&hdr_WWW_Authenticate->opaque);
	}
	break;
	case 9:
/* #line 90 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		hdr_WWW_Authenticate->stale = tsk_strniequals(tag_start, "true", 4);
	}
	break;
	case 10:
/* #line 94 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->algorithm);
	}
	break;
	case 11:
/* #line 98 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_SET_STRING(hdr_WWW_Authenticate->qop);
		//tsk_strunquote(&hdr_WWW_Authenticate->qop);
	}
	break;
	case 12:
/* #line 103 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
		TSK_PARSER_ADD_PARAM(THTTP_HEADER_PARAMS(hdr_WWW_Authenticate));
	}
	break;
	case 13:
/* #line 107 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	{
	}
	break;
/* #line 4610 "./src/headers/thttp_header_WWW_Authenticate.c" */
		}
	}

_again:
	if ( cs == 0 )
		goto _out;
	if ( ++p != pe )
		goto _resume;
	_test_eof: {}
	_out: {}
	}

/* #line 188 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
	
	if( cs < 
/* #line 4626 "./src/headers/thttp_header_WWW_Authenticate.c" */
926
/* #line 189 "./ragel/thttp_parser_header_WWW_Authenticate.rl" */
 ){
		TSK_DEBUG_ERROR("Failed to parse WWW-Authenticate header.");
		TSK_OBJECT_SAFE_FREE(hdr_WWW_Authenticate);
	}
	
	return hdr_WWW_Authenticate;
}

thttp_header_Proxy_Authenticate_t *thttp_header_Proxy_Authenticate_parse(const char *data, tsk_size_t size)
{
	return thttp_header_WWW_Authenticate_parse(data, size);
}





//========================================================
//	WWW_Authenticate header object definition
//

static tsk_object_t* thttp_header_WWW_Authenticate_ctor(tsk_object_t *self, va_list * app)
{
	thttp_header_WWW_Authenticate_t *WWW_Authenticate = self;
	if(WWW_Authenticate){
		THTTP_HEADER(WWW_Authenticate)->type = thttp_htype_WWW_Authenticate;
		THTTP_HEADER(WWW_Authenticate)->tostring = thttp_header_WWW_Authenticate_tostring;
	}
	else{
		TSK_DEBUG_ERROR("Failed to create new WWW_Authenticate header.");
	}
	return self;
}

static tsk_object_t* thttp_header_WWW_Authenticate_dtor(tsk_object_t *self)
{
	thttp_header_WWW_Authenticate_t *WWW_Authenticate = self;
	if(WWW_Authenticate){
		TSK_FREE(WWW_Authenticate->scheme);
		TSK_FREE(WWW_Authenticate->realm);
		TSK_FREE(WWW_Authenticate->domain);
		TSK_FREE(WWW_Authenticate->nonce);
		TSK_FREE(WWW_Authenticate->opaque);
		TSK_FREE(WWW_Authenticate->algorithm);
		TSK_FREE(WWW_Authenticate->qop);

		TSK_OBJECT_SAFE_FREE(THTTP_HEADER_PARAMS(WWW_Authenticate));
	}
	else{
		TSK_DEBUG_ERROR("Null WWW_Authenticate header.");
	}

	return self;
}

static const tsk_object_def_t thttp_header_WWW_Authenticate_def_s = 
{
	sizeof(thttp_header_WWW_Authenticate_t),
	thttp_header_WWW_Authenticate_ctor,
	thttp_header_WWW_Authenticate_dtor,
	tsk_null
};
const tsk_object_def_t *thttp_header_WWW_Authenticate_def_t = &thttp_header_WWW_Authenticate_def_s;
